
from .visitors import Visitable
from .annotation import Annotated
from .base import Executable, Immutable
from .operators import ColumnOperators
from .. import util

class ClauseElement(Visitable): ...

class ColumnElement(ColumnOperators, ClauseElement): ...

class BindParameter(ColumnElement): ...
class BinaryExpression(ColumnElement): ...

class TypeClause(ClauseElement): ...
class TextClause(Executable, ClauseElement): ...

class Null(ColumnElement): ...
class False_(ColumnElement): ...
class True_(ColumnElement): ...

class ClauseList(ClauseElement): ...
class BooleanClauseList(ClauseList, ColumnElement): ...
class Tuple(ClauseList, ColumnElement): ...
class Case(ColumnElement): ...
class Cast(ColumnElement): ...
class Extract(ColumnElement): ...
class _label_reference(ColumnElement): ...

class _textual_label_reference(ColumnElement): ...
class UnaryExpression(ColumnElement): ...
class AsBoolean(UnaryExpression): ...
class Grouping(ColumnElement): ...
class Over(ColumnElement): ...
class FunctionFilter(ColumnElement): ...
class Label(ColumnElement): ...
class ColumnClause(Immutable, ColumnElement): ...
class _IdentifiedClause(Executable, ClauseElement): ...
class SavepointClause(_IdentifiedClause): ...
class RollbackToSavepointClause(_IdentifiedClause): ...
class ReleaseSavepointClause(_IdentifiedClause): ...
class quoted_name(util.MemoizedSlots, util.text_type): ...
class _truncated_label(quoted_name): ...
class conv(_truncated_label): ...
class _defer_name(_truncated_label): ...
class _defer_none_name(_defer_name): ...
class _anonymous_label(_truncated_label): ...
class AnnotatedColumnElement(Annotated): ...

def _clone(element, **kw): ...
def _type_from_args(args): ...
def _literal_as_binds(element, name, type_=None): ...

def collate(expression, collation) -> BinaryExpression: ...
def between(expr, lower_bound, upper_bound, symmetric: bool=...): ...
def literal(value, type_=None) -> BindParameter: ...
def outparam(key, type_=None) -> BindParameter: ...
def type_coerce(expression, type_): ...
def not_(clause): ...
def literal_column(text, type_=None): ...



# Stubs for hmac (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
from _operator import _compare_digest as compare_digest

trans_5C = ... # type: Any
trans_36 = ... # type: Any
digest_size = ... # type: Any

class HMAC:
    blocksize = ... # type: Any
    digest_cons = ... # type: Any
    outer = ... # type: Any
    inner = ... # type: Any
    digest_size = ... # type: Any
    block_size = ... # type: Any
    def __init__(self, key, msg=None, digestmod=None): ...
    @property
    def name(self): ...
    def update(self, msg): ...
    def copy(self): ...
    def digest(self): ...
    def hexdigest(self): ...

def new(key, msg=None, digestmod=None): ...



# Stubs for html.entities (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

name2codepoint = ... # type: Any
html5 = ... # type: Any
codepoint2name = ... # type: Any
entitydefs = ... # type: Any



# Stubs for email._header_value_parser (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

WSP = ...  # type: Any
CFWS_LEADER = ...  # type: Any
SPECIALS = ...  # type: Any
ATOM_ENDS = ...  # type: Any
DOT_ATOM_ENDS = ...  # type: Any
PHRASE_ENDS = ...  # type: Any
TSPECIALS = ...  # type: Any
TOKEN_ENDS = ...  # type: Any
ASPECIALS = ...  # type: Any
ATTRIBUTE_ENDS = ...  # type: Any
EXTENDED_ATTRIBUTE_ENDS = ...  # type: Any

def quote_string(value): ...

class _Folded:
    maxlen = ...  # type: Any
    policy = ...  # type: Any
    lastlen = ...  # type: Any
    stickyspace = ...  # type: Any
    firstline = ...  # type: Any
    done = ...  # type: Any
    current = ...  # type: Any
    def __init__(self, maxlen, policy) -> None: ...
    def newline(self): ...
    def finalize(self): ...
    def append(self, stoken): ...
    def append_if_fits(self, token, stoken=...): ...

class TokenList(list):
    token_type = ...  # type: Any
    defects = ...  # type: Any
    def __init__(self, *args, **kw) -> None: ...
    @property
    def value(self): ...
    @property
    def all_defects(self): ...
    @property
    def parts(self): ...
    def startswith_fws(self): ...
    def pop_leading_fws(self): ...
    def pop_trailing_ws(self): ...
    @property
    def has_fws(self): ...
    def has_leading_comment(self): ...
    @property
    def comments(self): ...
    def fold(self, policy): ...
    def as_encoded_word(self, charset): ...
    def cte_encode(self, charset, policy): ...
    def pprint(self, indent=...): ...
    def ppstr(self, indent=...): ...

class WhiteSpaceTokenList(TokenList):
    @property
    def value(self): ...
    @property
    def comments(self): ...

class UnstructuredTokenList(TokenList):
    token_type = ...  # type: Any
    def cte_encode(self, charset, policy): ...

class Phrase(TokenList):
    token_type = ...  # type: Any
    def cte_encode(self, charset, policy): ...

class Word(TokenList):
    token_type = ...  # type: Any

class CFWSList(WhiteSpaceTokenList):
    token_type = ...  # type: Any
    def has_leading_comment(self): ...

class Atom(TokenList):
    token_type = ...  # type: Any

class Token(TokenList):
    token_type = ...  # type: Any

class EncodedWord(TokenList):
    token_type = ...  # type: Any
    cte = ...  # type: Any
    charset = ...  # type: Any
    lang = ...  # type: Any
    @property
    def encoded(self): ...

class QuotedString(TokenList):
    token_type = ...  # type: Any
    @property
    def content(self): ...
    @property
    def quoted_value(self): ...
    @property
    def stripped_value(self): ...

class BareQuotedString(QuotedString):
    token_type = ...  # type: Any
    @property
    def value(self): ...

class Comment(WhiteSpaceTokenList):
    token_type = ...  # type: Any
    def quote(self, value): ...
    @property
    def content(self): ...
    @property
    def comments(self): ...

class AddressList(TokenList):
    token_type = ...  # type: Any
    @property
    def addresses(self): ...
    @property
    def mailboxes(self): ...
    @property
    def all_mailboxes(self): ...

class Address(TokenList):
    token_type = ...  # type: Any
    @property
    def display_name(self): ...
    @property
    def mailboxes(self): ...
    @property
    def all_mailboxes(self): ...

class MailboxList(TokenList):
    token_type = ...  # type: Any
    @property
    def mailboxes(self): ...
    @property
    def all_mailboxes(self): ...

class GroupList(TokenList):
    token_type = ...  # type: Any
    @property
    def mailboxes(self): ...
    @property
    def all_mailboxes(self): ...

class Group(TokenList):
    token_type = ...  # type: Any
    @property
    def mailboxes(self): ...
    @property
    def all_mailboxes(self): ...
    @property
    def display_name(self): ...

class NameAddr(TokenList):
    token_type = ...  # type: Any
    @property
    def display_name(self): ...
    @property
    def local_part(self): ...
    @property
    def domain(self): ...
    @property
    def route(self): ...
    @property
    def addr_spec(self): ...

class AngleAddr(TokenList):
    token_type = ...  # type: Any
    @property
    def local_part(self): ...
    @property
    def domain(self): ...
    @property
    def route(self): ...
    @property
    def addr_spec(self): ...

class ObsRoute(TokenList):
    token_type = ...  # type: Any
    @property
    def domains(self): ...

class Mailbox(TokenList):
    token_type = ...  # type: Any
    @property
    def display_name(self): ...
    @property
    def local_part(self): ...
    @property
    def domain(self): ...
    @property
    def route(self): ...
    @property
    def addr_spec(self): ...

class InvalidMailbox(TokenList):
    token_type = ...  # type: Any
    @property
    def display_name(self): ...
    local_part = ...  # type: Any

class Domain(TokenList):
    token_type = ...  # type: Any
    @property
    def domain(self): ...

class DotAtom(TokenList):
    token_type = ...  # type: Any

class DotAtomText(TokenList):
    token_type = ...  # type: Any

class AddrSpec(TokenList):
    token_type = ...  # type: Any
    @property
    def local_part(self): ...
    @property
    def domain(self): ...
    @property
    def value(self): ...
    @property
    def addr_spec(self): ...

class ObsLocalPart(TokenList):
    token_type = ...  # type: Any

class DisplayName(Phrase):
    token_type = ...  # type: Any
    @property
    def display_name(self): ...
    @property
    def value(self): ...

class LocalPart(TokenList):
    token_type = ...  # type: Any
    @property
    def value(self): ...
    @property
    def local_part(self): ...

class DomainLiteral(TokenList):
    token_type = ...  # type: Any
    @property
    def domain(self): ...
    @property
    def ip(self): ...

class MIMEVersion(TokenList):
    token_type = ...  # type: Any
    major = ...  # type: Any
    minor = ...  # type: Any

class Parameter(TokenList):
    token_type = ...  # type: Any
    sectioned = ...  # type: Any
    extended = ...  # type: Any
    charset = ...  # type: Any
    @property
    def section_number(self): ...
    @property
    def param_value(self): ...

class InvalidParameter(Parameter):
    token_type = ...  # type: Any

class Attribute(TokenList):
    token_type = ...  # type: Any
    @property
    def stripped_value(self): ...

class Section(TokenList):
    token_type = ...  # type: Any
    number = ...  # type: Any

class Value(TokenList):
    token_type = ...  # type: Any
    @property
    def stripped_value(self): ...

class MimeParameters(TokenList):
    token_type = ...  # type: Any
    @property
    def params(self): ...

class ParameterizedHeaderValue(TokenList):
    @property
    def params(self): ...
    @property
    def parts(self): ...

class ContentType(ParameterizedHeaderValue):
    token_type = ...  # type: Any
    maintype = ...  # type: Any
    subtype = ...  # type: Any

class ContentDisposition(ParameterizedHeaderValue):
    token_type = ...  # type: Any
    content_disposition = ...  # type: Any

class ContentTransferEncoding(TokenList):
    token_type = ...  # type: Any
    cte = ...  # type: Any

class HeaderLabel(TokenList):
    token_type = ...  # type: Any

class Header(TokenList):
    token_type = ...  # type: Any

class Terminal(str):
    token_type = ...  # type: Any
    defects = ...  # type: Any
    def __new__(cls, value, token_type): ...
    @property
    def all_defects(self): ...
    def cte_encode(self, charset, policy): ...
    def pop_trailing_ws(self): ...
    def pop_leading_fws(self): ...
    @property
    def comments(self): ...
    def has_leading_comment(self): ...
    def __getnewargs__(self): ...

class WhiteSpaceTerminal(Terminal):
    @property
    def value(self): ...
    def startswith_fws(self): ...
    has_fws = ...  # type: Any

class ValueTerminal(Terminal):
    @property
    def value(self): ...
    def startswith_fws(self): ...
    has_fws = ...  # type: Any
    def as_encoded_word(self, charset): ...

class EWWhiteSpaceTerminal(WhiteSpaceTerminal):
    @property
    def value(self): ...
    @property
    def encoded(self): ...
    has_fws = ...  # type: Any

DOT = ...  # type: Any
ListSeparator = ...  # type: Any
RouteComponentMarker = ...  # type: Any

def get_fws(value): ...
def get_encoded_word(value): ...
def get_unstructured(value): ...
def get_qp_ctext(value): ...
def get_qcontent(value): ...
def get_atext(value): ...
def get_bare_quoted_string(value): ...
def get_comment(value): ...
def get_cfws(value): ...
def get_quoted_string(value): ...
def get_atom(value): ...
def get_dot_atom_text(value): ...
def get_dot_atom(value): ...
def get_word(value): ...
def get_phrase(value): ...
def get_local_part(value): ...
def get_obs_local_part(value): ...
def get_dtext(value): ...
def get_domain_literal(value): ...
def get_domain(value): ...
def get_addr_spec(value): ...
def get_obs_route(value): ...
def get_angle_addr(value): ...
def get_display_name(value): ...
def get_name_addr(value): ...
def get_mailbox(value): ...
def get_invalid_mailbox(value, endchars): ...
def get_mailbox_list(value): ...
def get_group_list(value): ...
def get_group(value): ...
def get_address(value): ...
def get_address_list(value): ...
def parse_mime_version(value): ...
def get_invalid_parameter(value): ...
def get_ttext(value): ...
def get_token(value): ...
def get_attrtext(value): ...
def get_attribute(value): ...
def get_extended_attrtext(value): ...
def get_extended_attribute(value): ...
def get_section(value): ...
def get_value(value): ...
def get_parameter(value): ...
def parse_mime_parameters(value): ...
def parse_content_type_header(value): ...
def parse_content_disposition_header(value): ...
def parse_content_transfer_encoding_header(value): ...



from typing import cast, List

from mypy.join import is_similar_callables, combine_similar_callables
from mypy.types import (
    Type, AnyType, TypeVisitor, UnboundType, Void, ErrorType, NoneTyp, TypeVarType,
    Instance, CallableType, TupleType, ErasedType, TypeList, UnionType, PartialType, DeletedType
)
from mypy.subtypes import is_subtype
from mypy.nodes import TypeInfo

# TODO Describe this module.


def meet_types(s: Type, t: Type) -> Type:
    """Return the greatest lower bound of two types."""
    if isinstance(s, ErasedType):
        return s
    if isinstance(s, AnyType):
        return t
    if isinstance(s, UnionType) and not isinstance(t, UnionType):
        s, t = t, s
    return t.accept(TypeMeetVisitor(s))


def meet_simple(s: Type, t: Type, default_right: bool = True) -> Type:
    if s == t:
        return s
    if isinstance(s, UnionType):
        return UnionType.make_simplified_union([meet_types(x, t) for x in s.items])
    elif not is_overlapping_types(s, t, use_promotions=True):
        return NoneTyp()
    else:
        if default_right:
            return t
        else:
            return s


def is_overlapping_types(t: Type, s: Type, use_promotions: bool = False) -> bool:
    """Can a value of type t be a value of type s, or vice versa?

    Note that this effectively checks against erased types, since type
    variables are erased at runtime and the overlapping check is based
    on runtime behavior.

    If use_promitions is True, also consider type promotions (int and
    float would only be overlapping if it's True).

    This does not consider multiple inheritance. For example, A and B in
    the following example are not considered overlapping, even though
    via C they can be overlapping:

        class A: ...
        class B: ...
        class C(A, B): ...

    The rationale is that this case is usually very unlikely as multiple
    inhreitance is rare. Also, we can't reliably determine whether
    multiple inheritance actually occurs somewhere in a program, due to
    stub files hiding implementation details, dynamic loading etc.

    TODO: Don't consider tuples always overlapping.
    TODO: Don't consider callables always overlapping.
    TODO: Don't consider type variables with values always overlapping.
    """
    # Since we are effectively working with the erased types, we only
    # need to handle occurrences of TypeVarType at the top level.
    if isinstance(t, TypeVarType):
        t = t.erase_to_union_or_bound()
    if isinstance(s, TypeVarType):
        s = s.erase_to_union_or_bound()
    if isinstance(t, Instance):
        if isinstance(s, Instance):
            # Only consider two classes non-disjoint if one is included in the mro
            # of another.
            if use_promotions:
                # Consider cases like int vs float to be overlapping where
                # there is only a type promotion relationship but not proper
                # subclassing.
                if t.type._promote and is_overlapping_types(t.type._promote, s):
                    return True
                if s.type._promote and is_overlapping_types(s.type._promote, t):
                    return True
            return t.type in s.type.mro or s.type in t.type.mro
    if isinstance(t, UnionType):
        return any(is_overlapping_types(item, s)
                   for item in t.items)
    if isinstance(s, UnionType):
        return any(is_overlapping_types(t, item)
                   for item in s.items)
    # We conservatively assume that non-instance, non-union types can overlap any other
    # types.
    return True


def nearest_builtin_ancestor(type: TypeInfo) -> TypeInfo:
    for base in type.mro:
        if base.defn.is_builtinclass:
            return base
    else:
        return None


class TypeMeetVisitor(TypeVisitor[Type]):
    def __init__(self, s: Type) -> None:
        self.s = s

    def visit_unbound_type(self, t: UnboundType) -> Type:
        if isinstance(self.s, Void) or isinstance(self.s, ErrorType):
            return ErrorType()
        elif isinstance(self.s, NoneTyp):
            return self.s
        else:
            return AnyType()

    def visit_error_type(self, t: ErrorType) -> Type:
        return t

    def visit_type_list(self, t: TypeList) -> Type:
        assert False, 'Not supported'

    def visit_any(self, t: AnyType) -> Type:
        return self.s

    def visit_union_type(self, t: UnionType) -> Type:
        if isinstance(self.s, UnionType):
            meets = []  # type: List[Type]
            for x in t.items:
                for y in self.s.items:
                    meets.append(meet_types(x, y))
        else:
            meets = [meet_types(x, self.s)
                     for x in t.items]
        return UnionType.make_simplified_union(meets)

    def visit_void(self, t: Void) -> Type:
        if isinstance(self.s, Void):
            return t
        else:
            return ErrorType()

    def visit_none_type(self, t: NoneTyp) -> Type:
        if not isinstance(self.s, Void) and not isinstance(self.s, ErrorType):
            return t
        else:
            return ErrorType()

    def visit_deleted_type(self, t: DeletedType) -> Type:
        if not isinstance(self.s, Void) and not isinstance(self.s, ErrorType):
            if isinstance(self.s, NoneTyp):
                return self.s
            else:
                return t
        else:
            return ErrorType()

    def visit_erased_type(self, t: ErasedType) -> Type:
        return self.s

    def visit_type_var(self, t: TypeVarType) -> Type:
        if isinstance(self.s, TypeVarType) and self.s.id == t.id:
            return self.s
        else:
            return self.default(self.s)

    def visit_instance(self, t: Instance) -> Type:
        if isinstance(self.s, Instance):
            si = self.s
            if t.type == si.type:
                if is_subtype(t, self.s) or is_subtype(self.s, t):
                    # Combine type arguments. We could have used join below
                    # equivalently.
                    args = []  # type: List[Type]
                    for i in range(len(t.args)):
                        args.append(self.meet(t.args[i], si.args[i]))
                    return Instance(t.type, args)
                else:
                    return NoneTyp()
            else:
                if is_subtype(t, self.s):
                    return t
                elif is_subtype(self.s, t):
                    # See also above comment.
                    return self.s
                else:
                    return NoneTyp()
        else:
            return self.default(self.s)

    def visit_callable_type(self, t: CallableType) -> Type:
        if isinstance(self.s, CallableType) and is_similar_callables(t, self.s):
            return combine_similar_callables(t, self.s)
        else:
            return self.default(self.s)

    def visit_tuple_type(self, t: TupleType) -> Type:
        if isinstance(self.s, TupleType) and self.s.length() == t.length():
            items = []  # type: List[Type]
            for i in range(t.length()):
                items.append(self.meet(t.items[i], self.s.items[i]))
            # TODO: What if the fallbacks are different?
            return TupleType(items, t.fallback)
        else:
            return self.default(self.s)

    def visit_partial_type(self, t: PartialType) -> Type:
        # We can't determine the meet of partial types. We should never get here.
        assert False, 'Internal error'

    def meet(self, s, t):
        return meet_types(s, t)

    def default(self, typ):
        if isinstance(typ, UnboundType):
            return AnyType()
        elif isinstance(typ, Void) or isinstance(typ, ErrorType):
            return ErrorType()
        else:
            return NoneTyp()



"""Find type annotations from a docstring.

Do not actually try to parse the annotations, just return them as strings.

Also recognize some common non-PEP-484 aliases such as 'a string' for 'str'
and 'list of int' for 'List[int]'.

Based on original implementation by Kyle Consalus.

TODO: Decide whether it makes sense to do the heuristic analysis of aliases and natural
  language type descriptions as it's all kind of ad hoc.
"""

import re
from typing import Optional, List, Tuple, Dict, Sequence
from collections import OrderedDict


_example1 = """Fetches rows from a Bigtable.

    Retrieves rows pertaining to the given keys from the Table instance
    represented by big_table.  Silly things may happen if
    other_silly_variable is not None.

    Args:
        big_table: An open Bigtable Table instance.
        keys (Sequence[str]): A sequence of strings representing the key of each table row
            to fetch.
            but: if the keys are broken, we die.
        other_silly_variable (int): Another optional variable, that has a much
            longer name than the other args, and which does nothing.
        abc0 (Tuple[int, bool]): Hi.
        abc (Tuple[int, bool], optional): Hi.

    Returns:
        Dict[str, int]: Things.

    Raises:
        IOError: An error occurred accessing the bigtable.Table object.
    """


# Regular expression that finds the argument name and type in a line such
# as '   name (type): description'.
PARAM_RE = re.compile(r'^\s*(?P<name>[A-Za-z_][A-Za-z_0-9]*)(\s+\((?P<type>[^)]+)\))?:')

# Type strings with these brackets are rejected.
BRACKET_RE = re.compile(r'\(|\)|\{|\}')

# Support some commonly used type aliases that aren't normally valid in annotations.
# TODO: Optionally reject these (or give a warning if these are used).
translations = {
    'obj': 'Any',
    'boolean': 'bool',
    'string': 'str',
    'integer': 'int',
    'number': 'float',
    'list': 'List[Any]',
    'set': 'Set[Any]',
    'sequence': 'Sequence[Any]',
    'iterable': 'Iterable[Any]',
    'dict': 'Dict[Any, Any]',
    'dictionary': 'Dict[Any, Any]',
    'mapping': 'Mapping[Any, Any]',
}

# Some common types that we should recognize.
known_types = [
    'int', 'str', 'unicode', 'bool', 'float', 'None', 'tuple',
]

known_generic_types = [
    'List', 'Set', 'Dict', 'Iterable', 'Sequence', 'Mapping',
]

# Some natural language patterns that we want to support in docstrings.
known_patterns = [
    ('list of ?', 'List[?]'),
    ('set of ?', 'List[?]'),
    ('sequence of ?', 'Sequence[?]'),
    ('optional ?', 'Optional[?]'),
]


class DocstringTypes(object):
    def __init__(self):
        self.args = OrderedDict()  # type: Dict[str, Optional[str]]
        self.rettype = None  # type: Optional[str]

    def as_type_str(self) -> str:
        return ('(' + ','.join([v or 'Any' for v in self.args.values()]) +
                ') -> ' + (self.rettype or 'Any'))

    def __str__(self):
        return repr({'args': self.args, 'return': self.rettype})


def wsprefix(s: str) -> str:
    return s[:len(s) - len(s.lstrip())]


def scrubtype(typestr: Optional[str], only_known=False) -> Optional[str]:
    if typestr is None:
        return typestr

    # Reject typestrs with parentheses or curly braces.
    if BRACKET_RE.search(typestr):
        return None

    # Reject typestrs whose square brackets don't match & those with commas outside square
    # brackets.
    bracket_level = 0
    for c in typestr:
        if c == '[':
            bracket_level += 1
        elif c == ']':
            bracket_level -= 1
            if bracket_level < 0:  # Square brackets don't match
                return None
        elif c == ',' and bracket_level == 0:  # A comma appears outside brackets
            return None
    if bracket_level > 0:
        return None

    recognized = False
    typestr = typestr.strip()
    for prefix in ('a', 'A', 'an', 'An', 'the', 'The'):
        if typestr.startswith(prefix + ' '):
            typestr = typestr[len(prefix) + 1:]
    if typestr in translations:
        typestr = translations[typestr]
        recognized = True
    if typestr in known_types:
        recognized = True
    if any(typestr.startswith(t + '[') for t in known_generic_types):
        recognized = True
    for pattern, repl in known_patterns:
        pattern = pattern.replace('?', '([a-zA-Z]+)') + '$'
        m = re.match(pattern, typestr)
        if m:
            arg = scrubtype(m.group(1), only_known=only_known)
            if arg:
                typestr = repl.replace('?', arg)
                recognized = True
    if not recognized and only_known:
        # This is potentially a type but not one of the known types.
        return None
    return typestr


def parse_args(lines: List[str]) -> Tuple[Dict[str, str], List[str]]:
    res = OrderedDict()  # type: Dict[str, str]
    indent = wsprefix(lines[0])
    while lines:
        l = lines[0]
        if l.strip() in ('Returns:', 'Raises:'):
            break
        lines = lines[1:]
        if not l or l.isspace():
            break
        if not wsprefix(l) == indent:
            continue
        m = PARAM_RE.match(l)
        if m:
            gd = m.groupdict()
            res[gd['name']] = scrubtype(gd['type'])
    return res, lines


def parse_return(lines: List[str]) -> Tuple[Optional[str], List[str]]:
    res = None  # type: Optional[str]
    while lines and lines[0].strip == '':
        lines = lines[1:]
    if lines:
        l = lines[0]
        lines = lines[1:]
        segs = l.strip().split(':', 1)
        if len(segs) >= 1:
            res = scrubtype(segs[0], only_known=(len(segs) == 1))
    return res, lines


def parse_docstring(pds: str) -> DocstringTypes:
    ds = DocstringTypes()
    lines = pds.splitlines()
    while lines:
        first = lines[0]
        if first.strip() in ('Args:', 'Params:', 'Arguments:'):
            ds.args, lines = parse_args(lines[1:])
            break
        lines = lines[1:]
    while lines:
        first = lines[0]
        if first.strip() == 'Returns:':
            ds.rettype, lines = parse_return(lines[1:])
            break
        lines = lines[1:]
    if not ds.args:
        return None
    return ds


if __name__ == '__main__':
    print(parse_docstring(_example1))



# Stubs for requests.packages.urllib3.connection (Python 3.4)

from typing import Any
from httplib import HTTPException
from . import packages
from . import exceptions
from . import util

class DummyConnection: ...

ConnectTimeoutError = exceptions.ConnectTimeoutError
SystemTimeWarning = exceptions.SystemTimeWarning
SecurityWarning = exceptions.SecurityWarning

port_by_scheme = ...  # type: Any
RECENT_DATE = ...  # type: Any

class HTTPConnection(object):
    default_port = ...  # type: Any
    default_socket_options = ...  # type: Any
    is_verified = ...  # type: Any
    source_address = ...  # type: Any
    socket_options = ...  # type: Any
    def __init__(self, *args, **kw) -> None: ...
    def connect(self): ...

class HTTPSConnection(HTTPConnection):
    default_port = ...  # type: Any
    key_file = ...  # type: Any
    cert_file = ...  # type: Any
    def __init__(self, host, port=..., key_file=..., cert_file=..., strict=..., timeout=..., **kw) -> None: ...
    sock = ...  # type: Any
    def connect(self): ...

class VerifiedHTTPSConnection(HTTPSConnection):
    cert_reqs = ...  # type: Any
    ca_certs = ...  # type: Any
    ssl_version = ...  # type: Any
    assert_fingerprint = ...  # type: Any
    key_file = ...  # type: Any
    cert_file = ...  # type: Any
    assert_hostname = ...  # type: Any
    def set_cert(self, key_file=..., cert_file=..., cert_reqs=..., ca_certs=..., assert_hostname=..., assert_fingerprint=...): ...
    sock = ...  # type: Any
    auto_open = ...  # type: Any
    is_verified = ...  # type: Any
    def connect(self): ...

UnverifiedHTTPSConnection = ...  # type: Any

class ConnectionError(Exception): pass



# Stubs for unicodedata (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

ucd_3_2_0 = ... # type: Any
ucnhash_CAPI = ... # type: Any
unidata_version = ... # type: str

def bidirectional(unichr): ...
def category(unichr): ...
def combining(unichr): ...
def decimal(chr, default=...): ...
def decomposition(unichr): ...
def digit(chr, default=...): ...
def east_asian_width(unichr): ...
def lookup(name): ...
def mirrored(unichr): ...
def name(chr, default=...): ...
def normalize(form, unistr): ...
def numeric(chr, default=...): ...

class UCD:
    unidata_version = ... # type: Any
    def bidirectional(self, unichr): ...
    def category(self, unichr): ...
    def combining(self, unichr): ...
    def decimal(self, chr, default=...): ...
    def decomposition(self, unichr): ...
    def digit(self, chr, default=...): ...
    def east_asian_width(self, unichr): ...
    def lookup(self, name): ...
    def mirrored(self, unichr): ...
    def name(self, chr, default=...): ...
    def normalize(self, form, unistr): ...
    def numeric(self, chr, default=...): ...



from typing import Any, AnyStr, Callable

class Operators(object):
    def op(self, opstring: AnyStr, precedence: int, is_comparison: bool): ...
    def operate(self, op: Callable[[Any], Any], *other, **kwargs): ...
    def reverse_operator(self, op: Callable[[Any], Any], *other, **kwargs): ...
    def __and__(self, other): ...
    def __or__(self, other): ...
    def __invert__(self): ...


class ColumnOperators(Operators):
    def concat(self, other): ...
    def like(self, other, escape=None): ...
    def ilike(self, other, escape=None): ...
    def notlike(self, other, escape=None): ...
    def notilike(self, other, escape=None): ...
    def in_(self, other): ...
    def notin_(self, other): ...
    def is_(self, other): ...
    def startswith(self, other, **kwargs): ...
    def endswith(self, other, **kwargs): ...
    def contains(self, other, **kwargs): ...
    def match(self, other, **kwargs): ...
    def desc(self): ...
    def asc(self): ...
    def nullsfirst(self): ...
    def nullslast(self): ...
    def collate(self, collation): ...
    def between(self, cleft, cright, symmetric: bool): ...
    def distinct(self): ...

    def __lt__(self, other): ...
    def __le__(self, other): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...
    def __gt__(self, other): ...
    def __ge__(self, other): ...
    def __neg__(self, other): ...
    def __getitem__(self, index): ...
    def __lshift__(self, other): ...
    def __rshift__(self, other): ...

    def __radd__(self, other): ...
    def __rsub__(self, other): ...
    def __rmul__(self, other): ...
    def __rdiv__(self, other): ...
    def __rmod__(self, other): ...
    def __add__(self, other): ...
    def __sub__(self, other): ...
    def __mul__(self, other): ...
    def __div__(self, other): ...
    def __mod__(self, other): ...
    def __truediv__(self, other): ...
    def __rtruediv__(self, other): ...

def from_(): ...
def as_(): ...
def exists(): ...
def istrue(a): ...
def isfalse(a): ...
def is_(a, b): ...
def isnot(a, b): ...
def collate(a, b): ...
def op(a, opstring, b): ...

def like_op(a, b, escape=None): ...
def notlike_op(a, b, escape=None): ...
def ilike_op(a, b, escape=None): ...
def notilike_op(a, b, escape=None): ...
def between_op(a, b, symmetric: bool): ...
def notbetween_(a, b, symmetric: bool): ...

def in_op(a, b): ...
def notin_op(a, b): ...
def distinct_op(a): ...

def startswith_op(a, b, escape=None): ...
def notstartswith_op(a, b, escape=None): ...
def endswith_op(a, b, escape=None): ...
def notendswith_op(a, b, escape=None): ...
def contains_op(a, b, escape=None): ...
def notcontains_op(a, b, escape=None): ...

def match_op(a, b, **kw): ...
def notmatch_op(a, b, **kw): ...

def comma_op(a, b): ...
def concat_op(a, b): ...

def desc_op(a): ...
def asc_op(a): ...
def nullsfirst_op(a): ...
def nullslast_op(a): ...

def is_comparison(op): ...
def is_commutative(op): ...
def is_ordering_modified(op): ...
def is_precedent(operator, against): ...



# TODO(MichalPokorny): more precise types

from typing import Any, Tuple, Optional

GLOBAL_SSL = ... # type: int
GLOBAL_WIN32 = ... # type: int
GLOBAL_ALL = ... # type: int
GLOBAL_NOTHING = ... # type: int
GLOBAL_DEFAULT = ... # type: int

def global_init(option: int) -> None: ...
def global_cleanup() -> None: ...

version = ... # type: str

def version_info() -> Tuple[int, str, int, str, int, str,
                            int, str, tuple, Any, int, Any]: ...

class error(Exception):
    pass

class Curl(object):
    def close(self) -> None: ...
    def setopt(self, option: int, value: Any) -> None: ...
    def perform(self) -> None: ...
    def getinfo(self, info: Any) -> Any: ...
    def reset(self) -> None: ...
    def unsetopt(self, option: int) -> Any: ...
    def pause(self, bitmask: Any) -> Any: ...
    def errstr(self) -> str: ...

    # TODO(MichalPokorny): wat?
    USERPWD = ... # type: int

class CurlMulti(object):
    def close(self) -> None: ...
    def add_handle(self, obj: Curl) -> None: ...
    def remove_handle(self, obj: Curl) -> None: ...
    def perform(self) -> Tuple[Any, int]: ...
    def fdset(self) -> tuple: ...
    def select(self, timeout: float = ...) -> int: ...
    def info_read(self, max_objects: int) -> tuple: ...

class CurlShare(object):
    def close(self) -> None: ...
    def setopt(self, option: int, value: Any) -> Any: ...

CAINFO = ... # type: int
CONNECTTIMEOUT_MS = ... # type: int
CUSTOMREQUEST = ... # type: int
ENCODING = ... # type: int
E_CALL_MULTI_PERFORM = ... # type: int
E_OPERATION_TIMEOUTED = ... # type: int
FOLLOWLOCATION = ... # type: int
HEADERFUNCTION = ... # type: int
HTTPGET = ... # type: int
HTTPHEADER = ... # type: int
HTTP_CODE = ... # type: int
INFILESIE_LARGE = ... # type: int
INFILESIZE_LARGE = ... # type: int
NOBODY = ... # type: int
NOPROGRESS = ... # type: int
NOSIGNAL = ... # type: int
POST = ... # type: int
POSTFIELDS = ... # type: int
POSTFIELDSIZE = ... # type: int
PRIMARY_IP = ... # type: int
PROGRESSFUNCTION = ... # type: int
PROXY = ... # type: int
READFUNCTION = ... # type: int
RESPONSE_CODE = ... # type: int
SSLCERT = ... # type: int
SSLCERTPASSWD = ... # type: int
SSLKEY = ... # type: int
SSLKEYPASSWD = ... # type: int
SSL_VERIFYHOST = ... # type: int
SSL_VERIFYPEER = ... # type: int
TIMEOUT_MS = ... # type: int
UPLOAD = ... # type: int
URL = ... # type: int
WRITEFUNCTION = ... # type: int



# Stubs for argparse (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Sequence, Union

SUPPRESS = ... # type: Any
OPTIONAL = ... # type: Any
ZERO_OR_MORE = ... # type: Any
ONE_OR_MORE = ... # type: Any
PARSER = ... # type: Any
REMAINDER = ... # type: Any

class _AttributeHolder: ...

class HelpFormatter:
    def __init__(self, prog, indent_increment=..., max_help_position=..., width=...) -> None: ...
    class _Section:
        formatter = ... # type: Any
        parent = ... # type: Any
        heading = ... # type: Any
        items = ... # type: Any
        def __init__(self, formatter, parent, heading=...) -> None: ...
        def format_help(self): ...
    def start_section(self, heading): ...
    def end_section(self): ...
    def add_text(self, text): ...
    def add_usage(self, usage, actions, groups, prefix=...): ...
    def add_argument(self, action): ...
    def add_arguments(self, actions): ...
    def format_help(self): ...

class RawDescriptionHelpFormatter(HelpFormatter): ...
class RawTextHelpFormatter(RawDescriptionHelpFormatter): ...
class ArgumentDefaultsHelpFormatter(HelpFormatter): ...

class ArgumentError(Exception):
    argument_name = ... # type: Any
    message = ... # type: Any
    def __init__(self, argument, message) -> None: ...

class ArgumentTypeError(Exception): ...

class Action(_AttributeHolder):
    option_strings = ... # type: Any
    dest = ... # type: Any
    nargs = ... # type: Any
    const = ... # type: Any
    default = ... # type: Any
    type = ... # type: Any
    choices = ... # type: Any
    required = ... # type: Any
    help = ... # type: Any
    metavar = ... # type: Any
    def __init__(self, option_strings, dest, nargs=..., const=..., default=..., type=...,
                 choices=..., required=..., help=..., metavar=...): ...
    def __call__(self, parser, namespace, values, option_string=...): ...

class _StoreAction(Action):
    def __init__(self, option_strings, dest, nargs=..., const=..., default=..., type=...,
                 choices=..., required=..., help=..., metavar=...): ...
    def __call__(self, parser, namespace, values, option_string=...): ...

class _StoreConstAction(Action):
    def __init__(self, option_strings, dest, const, default=..., required=..., help=...,
                 metavar=...): ...
    def __call__(self, parser, namespace, values, option_string=...): ...

class _StoreTrueAction(_StoreConstAction):
    def __init__(self, option_strings, dest, default=..., required=..., help=...) -> None: ...

class _StoreFalseAction(_StoreConstAction):
    def __init__(self, option_strings, dest, default=..., required=..., help=...) -> None: ...

class _AppendAction(Action):
    def __init__(self, option_strings, dest, nargs=..., const=..., default=..., type=...,
                 choices=..., required=..., help=..., metavar=...): ...
    def __call__(self, parser, namespace, values, option_string=...): ...

class _AppendConstAction(Action):
    def __init__(self, option_strings, dest, const, default=..., required=..., help=...,
                 metavar=...): ...
    def __call__(self, parser, namespace, values, option_string=...): ...

class _CountAction(Action):
    def __init__(self, option_strings, dest, default=..., required=..., help=...) -> None: ...
    def __call__(self, parser, namespace, values, option_string=...): ...

class _HelpAction(Action):
    def __init__(self, option_strings, dest=..., default=..., help=...) -> None: ...
    def __call__(self, parser, namespace, values, option_string=...): ...

class _VersionAction(Action):
    version = ... # type: Any
    def __init__(self, option_strings, version=..., dest=..., default=..., help=...) -> None: ...
    def __call__(self, parser, namespace, values, option_string=...): ...

class _SubParsersAction(Action):
    class _ChoicesPseudoAction(Action):
        def __init__(self, name, help) -> None: ...
    def __init__(self, option_strings, prog, parser_class, dest=..., help=..., metavar=...) -> None: ...
    def add_parser(self, name, **kwargs): ...
    def __call__(self, parser, namespace, values, option_string=...): ...

class FileType:
    def __init__(self, mode=..., bufsize=...) -> None: ...
    def __call__(self, string): ...

class Namespace(_AttributeHolder):
    def __init__(self, **kwargs) -> None: ...
    __hash__ = ... # type: Any
    def __eq__(self, other): ...
    def __ne__(self, other): ...
    def __contains__(self, key): ...
    def __getattr__(self, name: str) -> Any: ...

class _ActionsContainer:
    description = ... # type: Any
    argument_default = ... # type: Any
    prefix_chars = ... # type: Any
    conflict_handler = ... # type: Any
    def __init__(self, description, prefix_chars, argument_default, conflict_handler) -> None: ...
    def register(self, registry_name, value, object): ...
    def set_defaults(self, **kwargs): ...
    def get_default(self, dest): ...
    def add_argument(self,
        *args: str,
        action: Union[str, Action] = ...,
        nargs: str = ...,
        const: Any = ...,
        default: Any = ...,
        type: Any = ...,
        choices: Any = ..., # TODO: Container?
        required: bool = ...,
        help: str = ...,
        metavar: str = ...,
        dest: str = ...,
        version: str = ...
    ) -> None: ...
    def add_argument_group(self, *args, **kwargs): ...
    def add_mutually_exclusive_group(self, **kwargs): ...

class _ArgumentGroup(_ActionsContainer):
    title = ... # type: Any
    def __init__(self, container, title=..., description=..., **kwargs) -> None: ...

class _MutuallyExclusiveGroup(_ArgumentGroup):
    required = ... # type: Any
    def __init__(self, container, required=...) -> None: ...

class ArgumentParser(_AttributeHolder, _ActionsContainer):
    prog = ... # type: Any
    usage = ... # type: Any
    epilog = ... # type: Any
    version = ... # type: Any
    formatter_class = ... # type: Any
    fromfile_prefix_chars = ... # type: Any
    add_help = ... # type: Any
    def __init__(self, prog=..., usage=..., description=..., epilog=..., version=...,
                 parents=..., formatter_class=..., prefix_chars=..., fromfile_prefix_chars=...,
                 argument_default=..., conflict_handler=..., add_help=...): ...
    def add_subparsers(self, **kwargs): ...
    def parse_args(self, args: Sequence[str] = ..., namespace=...): ...
    def parse_known_args(self, args=..., namespace=...): ...
    def convert_arg_line_to_args(self, arg_line): ...
    def format_usage(self): ...
    def format_help(self): ...
    def format_version(self): ...
    def print_usage(self, file=...): ...
    def print_help(self, file=...): ...
    def print_version(self, file=...): ...
    def exit(self, status=..., message=...): ...
    def error(self, message): ...



# Stubs for errno

# Based on http://docs.python.org/3.2/library/errno.html

from typing import Dict

errorcode = ...  # type: Dict[int, str]

# TODO some of the names below are platform specific

EPERM = 0
ENOENT = 0
ESRCH = 0
EINTR = 0
EIO = 0
ENXIO = 0
E2BIG = 0
ENOEXEC = 0
EBADF = 0
ECHILD = 0
EAGAIN = 0
ENOMEM = 0
EACCES = 0
EFAULT = 0
ENOTBLK = 0
EBUSY = 0
EEXIST = 0
EXDEV = 0
ENODEV = 0
ENOTDIR = 0
EISDIR = 0
EINVAL = 0
ENFILE = 0
EMFILE = 0
ENOTTY = 0
ETXTBSY = 0
EFBIG = 0
ENOSPC = 0
ESPIPE = 0
EROFS = 0
EMLINK = 0
EPIPE = 0
EDOM = 0
ERANGE = 0
EDEADLK = 0
ENAMETOOLONG = 0
ENOLCK = 0
ENOSYS = 0
ENOTEMPTY = 0
ELOOP = 0
EWOULDBLOCK = 0
ENOMSG = 0
EIDRM = 0
ECHRNG = 0
EL2NSYNC = 0
EL3HLT = 0
EL3RST = 0
ELNRNG = 0
EUNATCH = 0
ENOCSI = 0
EL2HLT = 0
EBADE = 0
EBADR = 0
EXFULL = 0
ENOANO = 0
EBADRQC = 0
EBADSLT = 0
EDEADLOCK = 0
EBFONT = 0
ENOSTR = 0
ENODATA = 0
ETIME = 0
ENOSR = 0
ENONET = 0
ENOPKG = 0
EREMOTE = 0
ENOLINK = 0
EADV = 0
ESRMNT = 0
ECOMM = 0
EPROTO = 0
EMULTIHOP = 0
EDOTDOT = 0
EBADMSG = 0
EOVERFLOW = 0
ENOTUNIQ = 0
EBADFD = 0
EREMCHG = 0
ELIBACC = 0
ELIBBAD = 0
ELIBSCN = 0
ELIBMAX = 0
ELIBEXEC = 0
EILSEQ = 0
ERESTART = 0
ESTRPIPE = 0
EUSERS = 0
ENOTSOCK = 0
EDESTADDRREQ = 0
EMSGSIZE = 0
EPROTOTYPE = 0
ENOPROTOOPT = 0
EPROTONOSUPPORT = 0
ESOCKTNOSUPPORT = 0
EOPNOTSUPP = 0
EPFNOSUPPORT = 0
EAFNOSUPPORT = 0
EADDRINUSE = 0
EADDRNOTAVAIL = 0
ENETDOWN = 0
ENETUNREACH = 0
ENETRESET = 0
ECONNABORTED = 0
ECONNRESET = 0
ENOBUFS = 0
EISCONN = 0
ENOTCONN = 0
ESHUTDOWN = 0
ETOOMANYREFS = 0
ETIMEDOUT = 0
ECONNREFUSED = 0
EHOSTDOWN = 0
EHOSTUNREACH = 0
EALREADY = 0
EINPROGRESS = 0
ESTALE = 0
EUCLEAN = 0
ENOTNAM = 0
ENAVAIL = 0
EISNAM = 0
EREMOTEIO = 0
EDQUOT = 0



"""Type checking of attribute access"""

from typing import cast, Callable, List, Optional

from mypy.types import (
    Type, Instance, AnyType, TupleType, CallableType, FunctionLike, TypeVarDef,
    Overloaded, TypeVarType, TypeTranslator, UnionType, PartialType, DeletedType, NoneTyp
)
from mypy.nodes import TypeInfo, FuncBase, Var, FuncDef, SymbolNode, Context
from mypy.nodes import ARG_POS, ARG_STAR, ARG_STAR2, function_type, Decorator, OverloadedFuncDef
from mypy.messages import MessageBuilder
from mypy.maptype import map_instance_to_supertype
from mypy.expandtype import expand_type_by_instance
from mypy.nodes import method_type, method_type_with_fallback
from mypy.semanal import self_type
from mypy import messages
from mypy import subtypes


def analyze_member_access(name: str, typ: Type, node: Context, is_lvalue: bool,
                          is_super: bool,
                          builtin_type: Callable[[str], Instance],
                          not_ready_callback: Callable[[str, Context], None],
                          msg: MessageBuilder, override_info: TypeInfo = None,
                          report_type: Type = None) -> Type:
    """Analyse attribute access.

    This is a general operation that supports various different variations:

      1. lvalue or non-lvalue access (i.e. setter or getter access)
      2. supertype access (when using super(); is_super == True and
         override_info should refer to the supertype)
    """
    report_type = report_type or typ
    if isinstance(typ, Instance):
        if name == '__init__' and not is_super:
            # Accessing __init__ in statically typed code would compromise
            # type safety unless used via super().
            msg.fail(messages.CANNOT_ACCESS_INIT, node)
            return AnyType()

        # The base object has an instance type.

        info = typ.type
        if override_info:
            info = override_info

        # Look up the member. First look up the method dictionary.
        method = info.get_method(name)
        if method:
            if method.is_property:
                assert isinstance(method, OverloadedFuncDef)
                method = cast(OverloadedFuncDef, method)
                return analyze_var(name, method.items[0].var, typ, info, node, is_lvalue, msg,
                                   not_ready_callback)
            if is_lvalue:
                msg.cant_assign_to_method(node)
            typ = map_instance_to_supertype(typ, method.info)
            if name == '__new__':
                # __new__ is special and behaves like a static method -- don't strip
                # the first argument.
                signature = function_type(method, builtin_type('builtins.function'))
            else:
                signature = method_type_with_fallback(method, builtin_type('builtins.function'))
            return expand_type_by_instance(signature, typ)
        else:
            # Not a method.
            return analyze_member_var_access(name, typ, info, node,
                                             is_lvalue, is_super, builtin_type,
                                             not_ready_callback, msg,
                                             report_type=report_type)
    elif isinstance(typ, AnyType):
        # The base object has dynamic type.
        return AnyType()
    elif isinstance(typ, UnionType):
        # The base object has dynamic type.
        msg.disable_type_names += 1
        results = [analyze_member_access(name, subtype, node, is_lvalue,
                                         is_super, builtin_type, not_ready_callback, msg)
                   for subtype in typ.items]
        msg.disable_type_names -= 1
        return UnionType.make_simplified_union(results)
    elif isinstance(typ, TupleType):
        # Actually look up from the fallback instance type.
        return analyze_member_access(name, typ.fallback, node, is_lvalue,
                                     is_super, builtin_type, not_ready_callback, msg)
    elif isinstance(typ, FunctionLike) and typ.is_type_obj():
        # Class attribute.
        # TODO super?
        ret_type = typ.items()[0].ret_type
        if isinstance(ret_type, TupleType):
            ret_type = ret_type.fallback
        if isinstance(ret_type, Instance):
            result = analyze_class_attribute_access(ret_type, name, node, is_lvalue,
                                                    builtin_type, not_ready_callback, msg)
            if result:
                return result
            # Look up from the 'type' type.
            return analyze_member_access(name, typ.fallback, node, is_lvalue, is_super,
                                         builtin_type, not_ready_callback, msg,
                                         report_type=report_type)
        else:
            assert False, 'Unexpected type {}'.format(repr(ret_type))
    elif isinstance(typ, FunctionLike):
        # Look up from the 'function' type.
        return analyze_member_access(name, typ.fallback, node, is_lvalue, is_super,
                                     builtin_type, not_ready_callback, msg,
                                     report_type=report_type)
    elif isinstance(typ, TypeVarType):
        return analyze_member_access(name, typ.upper_bound, node, is_lvalue, is_super,
                                     builtin_type, not_ready_callback, msg,
                                     report_type=report_type)
    elif isinstance(typ, DeletedType):
        msg.deleted_as_rvalue(typ, node)
        return AnyType()
    return msg.has_no_attr(report_type, name, node)


def analyze_member_var_access(name: str, itype: Instance, info: TypeInfo,
                              node: Context, is_lvalue: bool, is_super: bool,
                              builtin_type: Callable[[str], Instance],
                              not_ready_callback: Callable[[str, Context], None],
                              msg: MessageBuilder,
                              report_type: Type = None) -> Type:
    """Analyse attribute access that does not target a method.

    This is logically part of analyze_member_access and the arguments are
    similar.
    """
    # It was not a method. Try looking up a variable.
    v = lookup_member_var_or_accessor(info, name, is_lvalue)

    vv = v
    if isinstance(vv, Decorator):
        # The associated Var node of a decorator contains the type.
        v = vv.var

    if isinstance(v, Var):
        return analyze_var(name, v, itype, info, node, is_lvalue, msg, not_ready_callback)
    elif isinstance(v, FuncDef):
        assert False, "Did not expect a function"
    elif not v and name not in ['__getattr__', '__setattr__']:
        if not is_lvalue:
            method = info.get_method('__getattr__')
            if method:
                typ = map_instance_to_supertype(itype, method.info)
                getattr_type = expand_type_by_instance(
                    method_type_with_fallback(method, builtin_type('builtins.function')), typ)
                if isinstance(getattr_type, CallableType):
                    return getattr_type.ret_type

    if itype.type.fallback_to_any:
        return AnyType()

    # Could not find the member.
    if is_super:
        msg.undefined_in_superclass(name, node)
        return AnyType()
    else:
        return msg.has_no_attr(report_type or itype, name, node)


def analyze_var(name: str, var: Var, itype: Instance, info: TypeInfo, node: Context,
               is_lvalue: bool, msg: MessageBuilder,
               not_ready_callback: Callable[[str, Context], None]) -> Type:
    """Analyze access to an attribute via a Var node.

    This is conceptually part of analyze_member_access and the arguments are similar.
    """
    # Found a member variable.
    itype = map_instance_to_supertype(itype, var.info)
    typ = var.type
    if typ:
        if isinstance(typ, PartialType):
            return handle_partial_attribute_type(typ, is_lvalue, msg, var)
        t = expand_type_by_instance(typ, itype)
        if var.is_initialized_in_class and isinstance(t, FunctionLike):
            if is_lvalue:
                if var.is_property:
                    if not var.is_settable_property:
                        msg.read_only_property(name, info, node)
                else:
                    msg.cant_assign_to_method(node)

            if not var.is_staticmethod:
                # Class-level function objects and classmethods become bound
                # methods: the former to the instance, the latter to the
                # class.
                functype = cast(FunctionLike, t)
                check_method_type(functype, itype, var.is_classmethod, node, msg)
                signature = method_type(functype)
                if var.is_property:
                    # A property cannot have an overloaded type => the cast
                    # is fine.
                    return cast(CallableType, signature).ret_type
                else:
                    return signature
        return t
    else:
        if not var.is_ready:
            not_ready_callback(var.name(), node)
        # Implicit 'Any' type.
        return AnyType()


def handle_partial_attribute_type(typ: PartialType, is_lvalue: bool, msg: MessageBuilder,
                                  context: Context) -> Type:
    if typ.type is None:
        # 'None' partial type. It has a well-defined type -- 'None'.
        # In an lvalue context we want to preserver the knowledge of
        # it being a partial type.
        if not is_lvalue:
            return NoneTyp()
        return typ
    else:
        msg.fail(messages.NEED_ANNOTATION_FOR_VAR, context)
        return AnyType()


def lookup_member_var_or_accessor(info: TypeInfo, name: str,
                                  is_lvalue: bool) -> SymbolNode:
    """Find the attribute/accessor node that refers to a member of a type."""
    # TODO handle lvalues
    node = info.get(name)
    if node:
        return node.node
    else:
        return None


def check_method_type(functype: FunctionLike, itype: Instance, is_classmethod: bool,
                      context: Context, msg: MessageBuilder) -> None:
    for item in functype.items():
        if not item.arg_types or item.arg_kinds[0] not in (ARG_POS, ARG_STAR):
            # No positional first (self) argument (*args is okay).
            msg.invalid_method_type(item, context)
        elif not is_classmethod:
            # Check that self argument has type 'Any' or valid instance type.
            selfarg = item.arg_types[0]
            if not subtypes.is_equivalent(selfarg, itype):
                msg.invalid_method_type(item, context)
        else:
            # Check that cls argument has type 'Any' or valid class type.
            # (This is sufficient for the current treatment of @classmethod,
            # but probably needs to be revisited when we implement Type[C]
            # or advanced variants of it like Type[<args>, C].)
            clsarg = item.arg_types[0]
            if isinstance(clsarg, CallableType) and clsarg.is_type_obj():
                if not subtypes.is_equivalent(clsarg.ret_type, itype):
                    msg.invalid_class_method_type(item, context)
            else:
                if not subtypes.is_equivalent(clsarg, AnyType()):
                    msg.invalid_class_method_type(item, context)


def analyze_class_attribute_access(itype: Instance,
                                   name: str,
                                   context: Context,
                                   is_lvalue: bool,
                                   builtin_type: Callable[[str], Instance],
                                   not_ready_callback: Callable[[str, Context], None],
                                   msg: MessageBuilder) -> Type:
    node = itype.type.get(name)
    if not node:
        if itype.type.fallback_to_any:
            return AnyType()
        return None

    is_decorated = isinstance(node.node, Decorator)
    is_method = is_decorated or isinstance(node.node, FuncDef)
    if is_lvalue:
        if is_method:
            msg.cant_assign_to_method(context)
        if isinstance(node.node, TypeInfo):
            msg.fail(messages.CANNOT_ASSIGN_TO_TYPE, context)

    if itype.type.is_enum and not (is_lvalue or is_decorated or is_method):
        return itype

    t = node.type
    if t:
        if isinstance(t, PartialType):
            return handle_partial_attribute_type(t, is_lvalue, msg, node.node)
        is_classmethod = is_decorated and cast(Decorator, node.node).func.is_class
        return add_class_tvars(t, itype.type, is_classmethod, builtin_type)
    elif isinstance(node.node, Var):
        not_ready_callback(name, context)
        return AnyType()

    if isinstance(node.node, TypeInfo):
        return type_object_type(node.node, builtin_type)

    if is_decorated:
        # TODO: Return type of decorated function. This is quick hack to work around #998.
        return AnyType()
    else:
        return function_type(cast(FuncBase, node.node), builtin_type('builtins.function'))


def add_class_tvars(t: Type, info: TypeInfo, is_classmethod: bool,
                    builtin_type: Callable[[str], Instance]) -> Type:
    if isinstance(t, CallableType):
        # TODO: Should we propagate type variable values?
        vars = [TypeVarDef(n, i + 1, None, builtin_type('builtins.object'), tv.variance)
                for (i, n), tv in zip(enumerate(info.type_vars), info.defn.type_vars)]
        arg_types = t.arg_types
        arg_kinds = t.arg_kinds
        arg_names = t.arg_names
        if is_classmethod:
            arg_types = arg_types[1:]
            arg_kinds = arg_kinds[1:]
            arg_names = arg_names[1:]
        return t.copy_modified(arg_types=arg_types, arg_kinds=arg_kinds, arg_names=arg_names,
                               variables=vars + t.variables)
    elif isinstance(t, Overloaded):
        return Overloaded([cast(CallableType, add_class_tvars(i, info, is_classmethod,
                                                              builtin_type))
                           for i in t.items()])
    return t


def type_object_type(info: TypeInfo, builtin_type: Callable[[str], Instance]) -> Type:
    """Return the type of a type object.

    For a generic type G with type variables T and S the type is generally of form

      Callable[..., G[T, S]]

    where ... are argument types for the __init__/__new__ method (without the self
    argument). Also, the fallback type will be 'type' instead of 'function'.
    """
    init_method = info.get_method('__init__')
    if not init_method:
        # Must be an invalid class definition.
        return AnyType()
    else:
        fallback = builtin_type('builtins.type')
        if init_method.info.fullname() == 'builtins.object':
            # No non-default __init__ -> look at __new__ instead.
            new_method = info.get_method('__new__')
            if new_method and new_method.info.fullname() != 'builtins.object':
                # Found one! Get signature from __new__.
                return type_object_type_from_function(new_method, info, fallback)
            # Both are defined by object.  But if we've got a bogus
            # base class, we can't know for sure, so check for that.
            if info.fallback_to_any:
                # Construct a universal callable as the prototype.
                sig = CallableType(arg_types=[AnyType(), AnyType()],
                                   arg_kinds=[ARG_STAR, ARG_STAR2],
                                   arg_names=["_args", "_kwds"],
                                   ret_type=AnyType(),
                                   fallback=builtin_type('builtins.function'))
                return class_callable(sig, info, fallback, None)
        # Construct callable type based on signature of __init__. Adjust
        # return type and insert type arguments.
        return type_object_type_from_function(init_method, info, fallback)


def type_object_type_from_function(init_or_new: FuncBase, info: TypeInfo,
                                   fallback: Instance) -> FunctionLike:
    signature = method_type_with_fallback(init_or_new, fallback)

    # The __init__ method might come from a generic superclass
    # (init_or_new.info) with type variables that do not map
    # identically to the type variables of the class being constructed
    # (info). For example
    #
    #   class A(Generic[T]): def __init__(self, x: T) -> None: pass
    #   class B(A[List[T]], Generic[T]): pass
    #
    # We need to first map B's __init__ to the type (List[T]) -> None.
    signature = cast(FunctionLike,
                     map_type_from_supertype(signature, info, init_or_new.info))

    if init_or_new.info.fullname() == 'builtins.dict':
        # Special signature!
        special_sig = 'dict'
    else:
        special_sig = None

    if isinstance(signature, CallableType):
        return class_callable(signature, info, fallback, special_sig)
    else:
        # Overloaded __init__/__new__.
        items = []  # type: List[CallableType]
        for item in cast(Overloaded, signature).items():
            items.append(class_callable(item, info, fallback, special_sig))
        return Overloaded(items)


def class_callable(init_type: CallableType, info: TypeInfo, type_type: Instance,
                   special_sig: Optional[str]) -> CallableType:
    """Create a type object type based on the signature of __init__."""
    variables = []  # type: List[TypeVarDef]
    for i, tvar in enumerate(info.defn.type_vars):
        variables.append(TypeVarDef(tvar.name, i + 1, tvar.values, tvar.upper_bound,
                                    tvar.variance))

    initvars = init_type.variables
    variables.extend(initvars)

    callable_type = init_type.copy_modified(
        ret_type=self_type(info), fallback=type_type, name=None, variables=variables,
        special_sig=special_sig)
    c = callable_type.with_name('"{}"'.format(info.name()))
    cc = convert_class_tvars_to_func_tvars(c, len(initvars))
    cc.is_classmethod_class = True
    return cc


def convert_class_tvars_to_func_tvars(callable: CallableType,
                                      num_func_tvars: int) -> CallableType:
    return cast(CallableType, callable.accept(TvarTranslator(num_func_tvars)))


class TvarTranslator(TypeTranslator):
    def __init__(self, num_func_tvars: int) -> None:
        super().__init__()
        self.num_func_tvars = num_func_tvars

    def visit_type_var(self, t: TypeVarType) -> Type:
        if t.id < 0:
            return t
        else:
            return TypeVarType(t.name, -t.id - self.num_func_tvars, t.values, t.upper_bound,
                               t.variance)

    def translate_variables(self,
                            variables: List[TypeVarDef]) -> List[TypeVarDef]:
        if not variables:
            return variables
        items = []  # type: List[TypeVarDef]
        for v in variables:
            if v.id > 0:
                items.append(TypeVarDef(v.name, -v.id - self.num_func_tvars,
                                        v.values, v.upper_bound, v.variance))
            else:
                items.append(v)
        return items


def map_type_from_supertype(typ: Type, sub_info: TypeInfo,
                            super_info: TypeInfo) -> Type:
    """Map type variables in a type defined in a supertype context to be valid
    in the subtype context. Assume that the result is unique; if more than
    one type is possible, return one of the alternatives.

    For example, assume

    . class D(Generic[S]) ...
    . class C(D[E[T]], Generic[T]) ...

    Now S in the context of D would be mapped to E[T] in the context of C.
    """
    # Create the type of self in subtype, of form t[a1, ...].
    inst_type = self_type(sub_info)
    if isinstance(inst_type, TupleType):
        inst_type = inst_type.fallback
    # Map the type of self to supertype. This gets us a description of the
    # supertype type variables in terms of subtype variables, i.e. t[t1, ...]
    # so that any type variables in tN are to be interpreted in subtype
    # context.
    inst_type = map_instance_to_supertype(inst_type, super_info)
    # Finally expand the type variables in type with those in the previously
    # constructed type. Note that both type and inst_type may have type
    # variables, but in type they are interpreterd in supertype context while
    # in inst_type they are interpreted in subtype context. This works even if
    # the names of type variables in supertype and subtype overlap.
    return expand_type_by_instance(typ, inst_type)



# Stubs for requests.structures (Python 3)

from typing import Any
import collections

class CaseInsensitiveDict(collections.MutableMapping):
    def __init__(self, data=..., **kwargs) -> None: ...
    def __setitem__(self, key, value): ...
    def __getitem__(self, key): ...
    def __delitem__(self, key): ...
    def __iter__(self): ...
    def __len__(self): ...
    def lower_items(self): ...
    def __eq__(self, other): ...
    def copy(self): ...

class LookupDict(dict):
    name = ...  # type: Any
    def __init__(self, name=...) -> None: ...
    def __getitem__(self, key): ...
    def get(self, key, default=...): ...



# Stubs for sqlalchemy.engine.strategies (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

from . import base

strategies = ... # type: Any

class EngineStrategy:
    def __init__(self) -> None: ...
    def create(self, *args, **kwargs): ...

class DefaultEngineStrategy(EngineStrategy):
    def create(self, name_or_url, **kwargs): ...

class PlainEngineStrategy(DefaultEngineStrategy):
    name = ... # type: Any
    engine_cls = ... # type: Any

class ThreadLocalEngineStrategy(DefaultEngineStrategy):
    name = ... # type: Any
    engine_cls = ... # type: Any

class MockEngineStrategy(EngineStrategy):
    name = ... # type: Any
    def create(self, name_or_url, executor, **kwargs): ...
    class MockConnection(base.Connectable):
        def __init__(self, dialect, execute) -> None: ...
        engine = ... # type: Any
        dialect = ... # type: Any
        name = ... # type: Any
        def contextual_connect(self, **kwargs): ...
        def execution_options(self, **kw): ...
        def compiler(self, statement, parameters, **kwargs): ...
        def create(self, entity, **kwargs): ...
        def drop(self, entity, **kwargs): ...
        def execute(self, object, *multiparams, **params): ...



from typing import List, Dict

import mypy.subtypes
from mypy.sametypes import is_same_type
from mypy.expandtype import expand_type
from mypy.types import Type, TypeVarType, CallableType, AnyType, Void
from mypy.messages import MessageBuilder
from mypy.nodes import Context


def apply_generic_arguments(callable: CallableType, types: List[Type],
                            msg: MessageBuilder, context: Context) -> Type:
    """Apply generic type arguments to a callable type.

    For example, applying [int] to 'def [T] (T) -> T' results in
    'def (int) -> int'.

    Note that each type can be None; in this case, it will not be applied.
    """
    tvars = callable.variables
    if len(tvars) != len(types):
        msg.incompatible_type_application(len(tvars), len(types), context)
        return AnyType()

    # Check that inferred type variable values are compatible with allowed
    # values and bounds.  Also, promote subtype values to allowed values.
    types = types[:]
    for i, type in enumerate(types):
        values = callable.variables[i].values
        if values and type:
            if isinstance(type, AnyType):
                continue
            if isinstance(type, TypeVarType) and type.values:
                # Allow substituting T1 for T if every allowed value of T1
                # is also a legal value of T.
                if all(any(is_same_type(v, v1) for v in values)
                       for v1 in type.values):
                    continue
            for value in values:
                if mypy.subtypes.is_subtype(type, value):
                    types[i] = value
                    break
            else:
                msg.incompatible_typevar_value(callable, i + 1, type, context)

        upper_bound = callable.variables[i].upper_bound
        if type and not mypy.subtypes.satisfies_upper_bound(type, upper_bound):
            msg.incompatible_typevar_value(callable, i + 1, type, context)

    # Create a map from type variable id to target type.
    id_to_type = {}  # type: Dict[int, Type]
    for i, tv in enumerate(tvars):
        if types[i]:
            id_to_type[tv.id] = types[i]

    # Apply arguments to argument types.
    arg_types = [expand_type(at, id_to_type) for at in callable.arg_types]

    # The callable may retain some type vars if only some were applied.
    remaining_tvars = [tv for tv in tvars if tv.id not in id_to_type]

    return callable.copy_modified(
        arg_types=arg_types,
        ret_type=expand_type(callable.ret_type, id_to_type),
        variables=remaining_tvars,
    )



# Stubs for thrift.protocol (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

# Names in __all__ with no definition:
#   TBase
#   TBinaryProtocol
#   TCompactProtocol
#   TJSONProtocol
#   TProtocol
#   fastbinary



# Generated by stubtool 0.1, DO NOT EDIT
# See https://github.com/o11c/stubtool
#
# Stubs for six.moves.urllib.error (Python 3.2)

from urllib.error import URLError as URLError
from urllib.error import HTTPError as HTTPError
from urllib.error import ContentTooShortError as ContentTooShortError



# Stubs for sqlalchemy.orm.session (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class _SessionClassMethods:
    @classmethod
    def close_all(cls): ...
    @classmethod
    def identity_key(cls, orm_util, *args, **kwargs): ...
    @classmethod
    def object_session(cls, instance): ...

class SessionTransaction:
    session = ... # type: Any
    nested = ... # type: Any
    def __init__(self, session, parent=..., nested=...) -> None: ...
    @property
    def is_active(self): ...
    def connection(self, bindkey, execution_options=..., **kwargs): ...
    def prepare(self): ...
    def commit(self): ...
    def rollback(self, _capture_exception=...): ...
    def close(self, invalidate=...): ...
    def __enter__(self): ...
    def __exit__(self, type, value, traceback): ...

class Session(_SessionClassMethods):
    public_methods = ... # type: Any
    identity_map = ... # type: Any
    bind = ... # type: Any
    transaction = ... # type: Any
    hash_key = ... # type: Any
    autoflush = ... # type: Any
    autocommit = ... # type: Any
    expire_on_commit = ... # type: Any
    twophase = ... # type: Any
    def __init__(self, bind=..., autoflush=..., expire_on_commit=..., _enable_transaction_accounting=..., autocommit=..., twophase=..., weak_identity_map=..., binds=..., extension=..., info=..., query_cls=...) -> None: ...
    connection_callable = ... # type: Any
    def info(self): ...
    def begin(self, subtransactions=..., nested=...): ...
    def begin_nested(self): ...
    def rollback(self): ...
    def commit(self): ...
    def prepare(self): ...
    def connection(self, mapper=..., clause=..., bind=..., close_with_result=..., execution_options=..., **kw): ...
    def execute(self, clause, params=..., mapper=..., bind=..., **kw): ...
    def scalar(self, clause, params=..., mapper=..., bind=..., **kw): ...
    def close(self): ...
    def invalidate(self): ...
    def expunge_all(self): ...
    def bind_mapper(self, mapper, bind): ...
    def bind_table(self, table, bind): ...
    def get_bind(self, mapper=..., clause=...): ...
    def query(self, *entities, **kwargs): ...
    @property
    def no_autoflush(self): ...
    def refresh(self, instance, attribute_names=..., lockmode=...): ...
    def expire_all(self): ...
    def expire(self, instance, attribute_names=...): ...
    def prune(self): ...
    def expunge(self, instance): ...
    def add(self, instance, _warn=...): ...
    def add_all(self, instances): ...
    def delete(self, instance): ...
    def merge(self, instance, load=...): ...
    def enable_relationship_loading(self, obj): ...
    def __contains__(self, instance): ...
    def __iter__(self): ...
    def flush(self, objects=...): ...
    def bulk_save_objects(self, objects, return_defaults=..., update_changed_only=...): ...
    def bulk_insert_mappings(self, mapper, mappings, return_defaults=...): ...
    def bulk_update_mappings(self, mapper, mappings): ...
    def is_modified(self, instance, include_collections=..., passive=...): ...
    @property
    def is_active(self): ...
    @property
    def dirty(self): ...
    @property
    def deleted(self): ...
    @property
    def new(self): ...

class sessionmaker(_SessionClassMethods):
    kw = ... # type: Any
    class_ = ... # type: Any
    def __init__(self, bind=..., class_=..., autoflush=..., autocommit=..., expire_on_commit=..., info=..., **kw) -> None: ...
    def __call__(self, **local_kw): ...
    def configure(self, **new_kw): ...

# Names in __all__ with no definition:
#   SessionExtension



# Stubs for hashlib

from abc import abstractmethod, ABCMeta
from typing import AbstractSet

class Hash(metaclass=ABCMeta):
    digest_size = ...  # type: int
    block_size = ...  # type: int

    # [Python documentation note] Changed in version 3.4: The name attribute has
    # been present in CPython since its inception, but until Python 3.4 was not
    # formally specified, so may not exist on some platforms
    name = ...  # type: str

    @abstractmethod
    def update(self, arg: bytes) -> None: ...
    @abstractmethod
    def digest(self) -> bytes: ...
    @abstractmethod
    def hexdigest(self) -> str: ...
    @abstractmethod
    def copy(self) -> 'Hash': ...

def md5(arg: bytes = ...) -> Hash: ...
def sha1(arg: bytes = ...) -> Hash: ...
def sha224(arg: bytes = ...) -> Hash: ...
def sha256(arg: bytes = ...) -> Hash: ...
def sha384(arg: bytes = ...) -> Hash: ...
def sha512(arg: bytes = ...) -> Hash: ...

def new(name: str, data: bytes = ...) -> Hash: ...

# New in version 3.2
algorithms_guaranteed = ...  # type: AbstractSet[str]
algorithms_available = ...  # type: AbstractSet[str]

# New in version 3.4
# TODO The documentation says "password and salt are interpreted as buffers of
# bytes", should we declare something other than bytes here?
def pbkdf2_hmac(name: str, password: bytes, salt: bytes, rounds: int, dklen: int = ...) -> bytes: ...



# Stubs for requests.status_codes (Python 3)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
from .structures import LookupDict

codes = ...  # type: Any



# Stubs for email.mime.multipart (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from email.mime.base import MIMEBase

class MIMEMultipart(MIMEBase):
    def __init__(self, _subtype=..., boundary=..., _subparts=..., **_params) -> None: ...



# Stubs for hmac (Python 2)

from typing import Any, AnyStr

class HMAC:
    def update(self, msg: str) -> None: ...
    def digest(self) -> str: ...
    def hexdigest(self) -> str: ...
    def copy(self) -> HMAC: ...

def new(key: str, msg: str = ..., digestmod: Any = ...) -> HMAC: ...

def compare_digest(a: AnyStr, b: AnyStr) -> bool: ...



from typing import Any, IO

class JSONDecodeError(ValueError):
    def dumps(self, obj: Any) -> str: ...
    def dump(self, obj: Any, fp: IO[str], *args: Any, **kwds: Any) -> None: ...
    def loads(self, s: str) -> Any: ...
    def load(self, fp: IO[str]) -> Any: ...



# Stubs for operator

from typing import Any, Callable, overload, Tuple

def __abs__(a: Any) -> Any: ...
def __add__(a: Any, b: Any) -> Any: ...
def __and__(a: Any, b: Any) -> Any: ...
def __concat__(a: Any, b: Any) -> Any: ...
def __contains__(container: Any, item: Any) -> bool: ...
def __delitem__(container: Any, item: Any) -> None: ...
def __delslice__(container: Any, b: int, c: int) -> None: ...
def __div__(a: Any, b: Any) -> Any: ...
def __eq__(a: Any, b: Any) -> Any: ...
def __floordiv__(a: Any, b: Any) -> Any: ...
def __ge__(a: Any, b: Any) -> Any: ...
def __getitem__(container: Any, key: Any) -> Any: ...
def __getslice__(container, b: int, c: int) -> Any: ...
def __gt__(a: Any, b: Any) -> Any: ...
def __iadd__(a: Any, b: Any) -> Any: ...
def __iand__(a: Any, b: Any) -> Any: ...
def __iconcat__(a: Any, b: Any) -> Any: ...
def __idiv__(a: Any, b: Any) -> Any: ...
def __ifloordiv__(a: Any, b: Any) -> Any: ...
def __ilshift__(a: Any, b: Any) -> Any: ...
def __imod__(a: Any, b: Any) -> Any: ...
def __imul__(a: Any, b: Any) -> Any: ...
def __index__(x: Any) -> Any: ...
def __inv__(x: Any) -> Any: ...
def __invert__(x: Any) -> Any: ...
def __ior__(a: Any, b: Any) -> Any: ...
def __ipow__(a: Any, b: Any) -> Any: ...
def __irepeat__(a: Any, b: int) -> Any: ...
def __irshift__(a: Any, b: Any) -> Any: ...
def __isub__(a: Any, b: Any) -> Any: ...
def __itruediv__(a: Any, b: Any) -> Any: ...
def __ixor__(a: Any, b: Any) -> Any: ...
def __le__(a: Any, b: Any) -> Any: ...
def __lshift__(a: Any, b: Any) -> Any: ...
def __lt__(a: Any, b: Any) -> Any: ...
def __mod__(a: Any, b: Any) -> Any: ...
def __mul__(a: Any, b: Any) -> Any: ...
def __ne__(a: Any, b: Any) -> Any: ...
def __neg__(x: Any) -> Any: ...
def __not__(x: Any) -> bool: ...
def __or__(a: Any, b: Any) -> Any: ...
def __pos__(x: Any) -> Any: ...
def __pow__(a: Any, b: Any) -> Any: ...
def __repeat__(a, b: int) -> Any: ...
def __rshift__(a: Any, b: Any) -> Any: ...
def __setitem__(container: Any, b: Any) -> None: ...
def __setslice__(container: Any, b: int, c: int, item: Any) -> None: ...
def __sub__(a: Any, b: Any) -> Any: ...
def __truediv__(a: Any, b: Any) -> Any: ...
def __xor__(a: Any, b: Any) -> Any: ...

def abs(x: Any) -> Any: ...
def add(a: Any, b: Any) -> Any: ...
def and_(a: Any, b: Any) -> Any: ...
def concat(a: Any, b: Any) -> Any: ...
def contains(container: Any, item: Any) -> bool: ...
def countOf(container: Any, item: Any) -> int: ...
def delitem(container: Any, item: Any) -> None: ...
def delslice(container: Any, b: int, c: int) -> None: ...
def div(a: Any, b: Any) -> Any: ...
def eq(a: Any, b: Any) -> Any: ...
def floordiv(a: Any, b: Any) -> Any: ...
def ge(a: Any, b: Any) -> Any: ...
def getitem(a: Any, b: Any) -> Any: ...
def getslice(container: Any, b: int, c: int) -> Any: ...
def gt(a: Any, b: Any) -> Any: ...
def iadd(a: Any, b: Any) -> Any: ...
def iand(a: Any, b: Any) -> Any: ...
def iconcat(a: Any, b: Any) -> Any: ...
def idiv(a: Any, b: Any) -> Any: ...
def ifloordiv(a: Any, b: Any) -> Any: ...
def ilshift(a: Any, b: Any) -> Any: ...
def imod(a: Any, b: Any) -> Any: ...
def imul(a: Any, b: Any) -> Any: ...
def index(x: Any) -> Any: ...
def indexOf(container: Any, item: Any) -> int: ...
def inv(x: Any) -> Any: ...
def invert(x: Any) -> Any: ...
def ior(a: Any, b: Any) -> Any: ...
def ipow(a: Any, b: Any) -> Any: ...
def irepeat(a, b: int) -> Any: ...
def irshift(a: Any, b: Any) -> Any: ...
def isCallable(x: Any) -> bool: ...
def isMappingType(x: Any) -> bool: ...
def isNumberType(x: Any) -> bool: ...
def isSequenceType(x: Any) -> bool: ...
def is_(a: Any, b: Any) -> bool: ...
def is_not(a: Any, b: Any) -> bool: ...
def isub(a: Any, b: Any) -> Any: ...
def itruediv(a: Any, b: Any) -> Any: ...
def ixor(a: Any, b: Any) -> Any: ...
def le(a: Any, b: Any) -> Any: ...
def lshift(a: Any, b: Any) -> Any: ...
def lt(a: Any, b: Any) -> Any: ...
def mod(a: Any, b: Any) -> Any: ...
def mul(a: Any, b: Any) -> Any: ...
def ne(a: Any, b: Any) -> Any: ...
def neg(x: Any) -> Any: ...
def not_(x: Any) -> bool: ...
def or_(a: Any, b: Any) -> Any: ...
def pos(x: Any) -> Any: ...
def pow(a: Any, b: Any) -> Any: ...
def repeat(a, b: int) -> Any: ...
def rshift(a: Any, b: Any) -> Any: ...
def sequenceIncludes(seq1: Any, seq2: Any) -> bool: ...
def setitem(container: Any, key: Any, item: Any) -> None: ...
def setslice(container: Any, b: int, c: int, slice: Any) -> None: ...
def sub(a: Any, b: Any) -> Any: ...
def truediv(a: Any, b: Any) -> Any: ...
def truth(x: Any) -> bool: ...
def xor(a: Any, b: Any) -> Any: ...

# Unsupported: more than 3 attributes.
# Unsupported: on Python 2 the parameter type should be `basestring`.
@overload
def attrgetter(attr1: str) -> Callable[[Any], Any]: ...
@overload
def attrgetter(attr1: str, attr2: str) -> Callable[[Any], Tuple[Any, Any]]: ...
@overload
def attrgetter(attr1: str, attr2: str, attr3: str) -> Callable[[Any], Tuple[Any, Any, Any]]: ...

# Unsupported: more than 3 items.
@overload
def itemgetter(item1: Any) -> Callable[[Any], Any]: ...
@overload
def itemgetter(item1: Any, item2: Any) -> Callable[[Any], Tuple[Any, Any]]: ...
@overload
def itemgetter(item1: Any, item2: Any, item3: Any) -> Callable[[Any], Tuple[Any, Any, Any]]: ...

# Unsupported: on Python 2 the parameter type should be `basestring`.
def methodcaller(name: str, *args, **kwargs) -> Callable[[Any], Any]: ...



# Stubs for curses (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from _curses import *
# Stubgen imports a python version of has_key only if it's not present
#in _curses (which it is in this stub)
# from .has_key import has_key as has_key

def initscr(): ...
def start_color(): ...
def wrapper(func, *args, **kwds): ...



# Stubs for email.encoders (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

def encode_base64(msg): ...
def encode_quopri(msg): ...
def encode_7or8bit(msg): ...
def encode_noop(msg): ...



from typing import Any, Iterable, Optional, Tuple
import datetime

LocaleType = Tuple[Optional[str], Optional[str]]

class IllegalMonthError(ValueError):
    def __init__(self, month: int) -> None: ...
    def __str__(self) -> str: ...

class IllegalWeekdayError(ValueError):
    def __init__(self, weekday: int) -> None: ...
    def __str__(self) -> str: ...

def isleap(year: int) -> bool: ...
def leapdays(y1: int, y2: int) -> int: ...
def weekday(year: int, month: int, day: int) -> int: ...
def monthrange(year: int, month: int) -> Tuple[int, int]: ...

class Calendar(object):
    def __init__(self, firstweekday: int = 0) -> None: ...
    def getfirstweekday(self) -> int: ...
    def setfirstweekday(self, firstweekday: int) -> None: ...
    def iterweekdays(self) -> Iterable[int]: ...
    def itermonthdates(self, year: int, month: int) -> Iterable[datetime.date]: ...
    def itermonthdays2(self, year: int, month: int) -> Iterable[Tuple[int, int]]: ...
    def itermonthdays(self, year: int, month: int) -> Iterable[int]: ...
    def monthdatescalendar(self, year: int, month: int) -> List[List[datetime.date]]: ...
    def monthdays2calendar(self, year: int, month: int) -> List[List[Tuple[int, int]]]: ...
    def monthdayscalendar(self, year: int, month: int) -> List[List[int]]: ...
    def yeardatescalendar(self, year: int, width: int = 3) -> List[List[int]]: ...
    def yeardays2calendar(self, year: int, width: int = 3) -> List[List[Tuple[int, int]]]: ...
    def yeardayscalendar(self, year: int, width: int = 3) -> List[List[int]]: ...

class TextCalendar(Calendar):
    def prweek(self, theweek: int, width: int) -> None: ...
    def formatday(self, day: int, weekday: int, width: int) -> str: ...
    def formatweek(self, theweek: int, width: int) -> str: ...
    def formatweekday(self, day: int, width: int) -> str: ...
    def formatweekheader(self, width: int) -> str: ...
    def formatmonthname(self, theyear: int, themonth: int, width: int, withyear: bool = ...) -> str: ...
    def prmonth(self, theyear: int, themonth: int, w: Any=0, l: Any = 0) -> None: ...
    def formatmonth(self, theyear: int, themonth: int, w: int = 0, l: int = 0) -> str: ...
    def formatyear(self, theyear: int, w: int = 2, l: int = 1, c: int = 6, m: int = 3) -> str: ...
    def pryear(self, theyear: int, w: Any = 0, l: Any = 0, c: Any = 6, m: Any = 3) -> None: ...

class HTMLCalendar(Calendar):
    def formatday(self, day: int, weekday: int) -> str: ...
    def formatweek(self, theweek: int) -> str: ...
    def formatweekday(self, day: int) -> str: ...
    def formatweekheader(self) -> str: ...
    def formatmonthname(self, theyear: int, themonth: int, withyear: bool = ...) -> str: ...
    def formatmonth(self, theyear: int, themonth: int, withyear: bool = ...) -> str: ...
    def formatyear(self, theyear: int, width: int = 3) -> str: ...
    def formatyearpage(self, theyear: int, width: int = 3, css: Optional[str] = 'calendar.css', encoding: Optional[str] = ...) -> str: ...

class TimeEncoding:
    def __init__(self, locale: LocaleType) -> None: ...
    def __enter__(self) -> LocaleType: ...
    def __exit__(self, *args) -> None: ...

class LocaleTextCalendar(TextCalendar):
    def __init__(self, firstweekday: int = 0, locale: Optional[LocaleType] = ...) -> None: ...
    def formatweekday(self, day: int, width: int) -> str: ...
    def formatmonthname(self, theyear: int, themonth: int, width: int, withyear: bool = ...) -> str: ...

class LocaleHTMLCalendar(HTMLCalendar):
    def __init__(self, firstweekday: int = 0, locale: Optional[LocaleType] = ...) -> None: ...
    def formatweekday(self, day: int) -> str: ...
    def formatmonthname(self, theyear: int, themonth: int, withyear: bool = ...) -> str: ...

c = ...  # type: TextCalendar
def setfirstweekday(firstweekday: int) -> None: ...
def format(cols: int, colwidth: int = ..., spacing: int = ...) -> str: ...
def formatstring(cols: int, colwidth: int = ..., spacing: int = ...) -> str: ...
def timegm(tuple: Tuple[int]) -> int: ...



# Stubs for sqlalchemy.sql.expression (Python 2)

from typing import Any
from . import functions
from . import elements
from . import base
from . import selectable
from . import dml

func = functions.func # type: functions._FunctionGenerator
modifier = functions.modifier # type: functions._FunctionGenerator

from .visitors import Visitable

from .elements import ClauseElement, ColumnElement,\
    BindParameter, UnaryExpression, BooleanClauseList, \
    Label, Cast, Case, ColumnClause, TextClause, Over, Null, \
    True_, False_, BinaryExpression, Tuple, TypeClause, Extract, \
    Grouping, not_, \
    collate, literal_column, between,\
    literal, outparam, type_coerce, ClauseList, FunctionFilter
from .elements import SavepointClause, RollbackToSavepointClause, \
    ReleaseSavepointClause
from .base import ColumnCollection, Generative, Executable
from .selectable import Alias, Join, Select, Selectable, TableClause, \
    CompoundSelect, CTE, FromClause, FromGrouping, SelectBase, \
    alias, GenerativeSelect, \
    subquery, HasPrefixes, HasSuffixes, Exists, ScalarSelect, TextAsFrom
from .dml import Insert, Update, Delete, UpdateBase, ValuesBase

and_ = ... # type: Any
or_ = ... # type: Any
bindparam = ... # type: Any
select = ... # type: Any
text = ... # type: Any
table = ... # type: Any
column = ... # type: Any
over = ... # type: Any
label = ... # type: Any
case = ... # type: Any
cast = ... # type: Any
extract = ... # type: Any
tuple_ = ... # type: Any
except_ = ... # type: Any
except_all = ... # type: Any
intersect = ... # type: Any
intersect_all = ... # type: Any
union = ... # type: Any
union_all = ... # type: Any
exists = ... # type: Any
nullsfirst = ... # type: Any
nullslast = ... # type: Any
asc = ... # type: Any
desc = ... # type: Any
distinct = ... # type: Any
true = ... # type: Any
false = ... # type: Any
null = ... # type: Any
join = ... # type: Any
outerjoin = ... # type: Any
insert = ... # type: Any
update = ... # type: Any
delete = ... # type: Any
funcfilter = ... # type: Any

# old names for compatibility
_Executable = Executable
_BindParamClause = BindParameter
_Label = Label
_SelectBase = SelectBase
_BinaryExpression = BinaryExpression
_Cast = Cast
_Null = Null
_False = False_
_True = True_
_TextClause = TextClause
_UnaryExpression = UnaryExpression
_Case = Case
_Tuple = Tuple
_Over = Over
_Generative = Generative
_TypeClause = TypeClause
_Extract = Extract
_Exists = Exists
_Grouping = Grouping
_FromGrouping = FromGrouping
_ScalarSelect = ScalarSelect



# Generated by stubtool 0.1, DO NOT EDIT
# See https://github.com/o11c/stubtool
#
# Stubs for six.moves.urllib (Python 2.7)

import six.moves.urllib.error as error
import six.moves.urllib.parse as parse
import six.moves.urllib.request as request
import six.moves.urllib.response as response
import six.moves.urllib.robotparser as robotparser



# Stubs for itertools

# Based on http://docs.python.org/3.2/library/itertools.html

from typing import (Iterator, TypeVar, Iterable, overload, Any, Callable, Tuple,
                    Union, Sequence, Generic)

_T = TypeVar('_T')
_S = TypeVar('_S')

def count(start: int = ...,
          step: int = ...) -> Iterator[int]: ... # more general types?
def cycle(iterable: Iterable[_T]) -> Iterator[_T]: ...

@overload
def repeat(object: _T) -> Iterator[_T]: ...
@overload
def repeat(object: _T, times: int) -> Iterator[_T]: ...

def accumulate(iterable: Iterable[_T]) -> Iterator[_T]: ...

class chain(Iterator[_T], Generic[_T]):
    def __init__(self, *iterables: Iterable[_T]) -> None: ...
    def __next__(self) -> _T: ...
    def __iter__(self) -> Iterator[_T]: ...
    @staticmethod
    def from_iterable(iterable: Iterable[Iterable[_S]]) -> Iterator[_S]: ...

def compress(data: Iterable[_T], selectors: Iterable[Any]) -> Iterator[_T]: ...
def dropwhile(predicate: Callable[[_T], Any],
              iterable: Iterable[_T]) -> Iterator[_T]: ...
def filterfalse(predicate: Callable[[_T], Any],
                iterable: Iterable[_T]) -> Iterator[_T]: ...

@overload
def groupby(iterable: Iterable[_T]) -> Iterator[Tuple[_T, Iterator[_T]]]: ...
@overload
def groupby(iterable: Iterable[_T],
            key: Callable[[_T], _S]) -> Iterator[Tuple[_S, Iterator[_T]]]: ...

@overload
def islice(iterable: Iterable[_T], stop: int) -> Iterator[_T]: ...
@overload
def islice(iterable: Iterable[_T], start: int, stop: int,
           step: int = ...) -> Iterator[_T]: ...

def starmap(func: Any, iterable: Iterable[Any]) -> Iterator[Any]: ...
def takewhile(predicate: Callable[[_T], Any],
              iterable: Iterable[_T]) -> Iterator[_T]: ...
def tee(iterable: Iterable[Any], n: int = ...) -> Iterator[Any]: ...
def zip_longest(*p: Iterable[Any],
                fillvalue: Any = ...) -> Iterator[Any]: ...

# TODO: Return type should be Iterator[Tuple[..]], but unknown tuple shape.
#       Iterator[Sequence[_T]] loses this type information.
def product(*p: Iterable[_T], repeat: int = ...) -> Iterator[Sequence[_T]]: ...

def permutations(iterable: Iterable[_T],
                 r: Union[int, None] = ...) -> Iterator[Sequence[_T]]: ...
def combinations(iterable: Iterable[_T],
                 r: int) -> Iterable[Sequence[_T]]: ...
def combinations_with_replacement(iterable: Iterable[_T],
                                  r: int) -> Iterable[Sequence[_T]]: ...



"""Lexical analyzer for mypy.

Translate a string that represents a file or a compilation unit to a list of
tokens.

This module can be run as a script (lex.py FILE).
"""

import re

from mypy.util import short_type, find_python_encoding
from mypy import defaults
from typing import List, Callable, Dict, Any, Match, Pattern, Set, Union, Tuple


class Token:
    """Base class for all tokens."""

    def __init__(self, string: str, pre: str = '') -> None:
        """Initialize a token.

        Arguments:
          string: Token string in program text
          pre:    Space, comments etc. before token
        """

        self.string = string
        self.pre = pre
        self.line = 0

    def __repr__(self) -> str:
        """The representation is of form 'Keyword(  if)'."""
        t = short_type(self)
        return t + '(' + self.fix(self.pre) + self.fix(self.string) + ')'

    def rep(self) -> str:
        return self.pre + self.string

    def fix(self, s: str) -> str:
        """Replace common non-printable chars with escape sequences.

        Do not use repr() since we don't want do duplicate backslashes.
        """
        return s.replace('\n', '\\n').replace('\t', '\\t').replace('\r', '\\r')


# Token classes


class Break(Token):
    """Statement break (line break or semicolon)"""


class Indent(Token):
    """Increase block indent level."""


class Dedent(Token):
    """Decrease block indent level."""


class Eof(Token):
    """End of file"""


class Keyword(Token):
    """Reserved word (other than keyword operators; they use Op).

    Examples: if, class, while, def.
    """


class Name(Token):
    """An alphanumeric identifier"""


class IntLit(Token):
    """Integer literal"""


class StrLit(Token):
    """String literal"""

    def parsed(self) -> str:
        """Return the parsed contents of the literal."""
        return _parse_str_literal(self.string)


class BytesLit(Token):
    """Bytes literal"""

    def parsed(self) -> str:
        """Return the parsed contents of the literal."""
        return _parse_str_literal(self.string)


class UnicodeLit(Token):
    """Unicode literal (Python 2.x)"""

    def parsed(self) -> str:
        """Return the parsed contents of the literal."""
        return _parse_str_literal(self.string)


class FloatLit(Token):
    """Float literal"""


class ComplexLit(Token):
    """Complex literal"""


class Punct(Token):
    """Punctuator (e.g. comma, '(' or '=')"""


class Colon(Token):
    pass


class EllipsisToken(Token):
    pass


class Op(Token):
    """Operator (e.g. '+' or 'in')"""


class Bom(Token):
    """Byte order mark (at the start of a file)"""


class LexError(Token):
    """Lexer error token"""

    def __init__(self, string: str, type: int, message: str = None) -> None:
        """Initialize token.

        The type argument is one of the error types below.
        """
        super().__init__(string)
        self.type = type
        self.message = message

    def __str__(self):
        if self.message:
            return 'LexError(%s)' % self.message
        else:
            return super().__str__()


# Lexer error types
NUMERIC_LITERAL_ERROR = 0
UNTERMINATED_STRING_LITERAL = 1
INVALID_CHARACTER = 2
DECODE_ERROR = 3
INVALID_BACKSLASH = 4
INVALID_DEDENT = 5


def lex(string: Union[str, bytes], first_line: int = 1,
        pyversion: Tuple[int, int] = defaults.PYTHON3_VERSION,
        is_stub_file: bool = False) -> Tuple[List[Token], Set[int]]:
    """Analyze string, and return an array of token objects and the lines to ignore.

    The last token is always Eof. The intention is to ignore any
    semantic and type check errors on the ignored lines.
    """
    l = Lexer(pyversion, is_stub_file=is_stub_file)
    l.lex(string, first_line)
    return l.tok, l.ignored_lines


# Reserved words (not including operators)
keywords_common = set([
    'as', 'assert', 'break', 'class', 'continue', 'def', 'del', 'elif',
    'else', 'except', 'finally', 'from', 'for', 'global', 'if', 'import',
    'lambda', 'pass', 'raise', 'return', 'try', 'while', 'with',
    'yield'])

# Reserved words specific for Python version 2
# TODO (jukka): 'print' should be here, but it breaks the parsing of Python 2
#               builtins, since they also define the function 'print'.
keywords2 = set([])  # type: Set[str]

# Reserved words specific for Python version 3
keywords3 = set(['nonlocal'])

# Alphabetical operators (reserved words)
alpha_operators = set(['in', 'is', 'not', 'and', 'or'])

# String literal prefixes
str_prefixes = set(['r', 'b', 'br', 'rb', 'u', 'ur', 'R', 'B', 'U'])

# List of regular expressions that match non-alphabetical operators
operators = [re.compile('[-+*/<>.%&|^~]'),
             re.compile('==|!=|<=|>=|\\*\\*|//|<<|>>|<>')]

# List of regular expressions that match punctuator tokens
punctuators = [re.compile('[=,()@`]|(->)'),
               re.compile('\\['),
               re.compile(']'),
               re.compile('([-+*/%&|^]|\\*\\*|//|<<|>>)=')]


# Map single-character string escape sequences to corresponding characters.
escape_map = {'a': '\x07',
              'b': '\x08',
              'f': '\x0c',
              'n': '\x0a',
              'r': '\x0d',
              't': '\x09',
              'v': '\x0b',
              '"': '"',
              "'": "'"}


# Matches the optional prefix of a string literal, e.g. the 'r' in r"foo".
str_prefix_re = re.compile('[rRbBuU]*')

# Matches an escape sequence in a string, e.g. \n or \x4F.
escape_re = re.compile(
    "\\\\([abfnrtv'\"]|x[0-9a-fA-F]{2}|u[0-9a-fA-F]{4}|[0-7]{1,3})")


def _parse_str_literal(string: str) -> str:
    """Translate escape sequences in str literal to the corresponding chars.

    For example, \t is translated to the tab character (ascii 9).

    Return the translated contents of the literal.  Also handle raw and
    triple-quoted string literals.
    """

    prefix = str_prefix_re.match(string).group(0).lower()
    s = string[len(prefix):]
    if s.startswith("'''") or s.startswith('"""'):
        return s[3:-3]
    elif 'r' in prefix or 'R' in prefix:
        return s[1:-1].replace('\\' + s[0], s[0])
    else:
        return escape_re.sub(lambda m: escape_repl(m, prefix), s[1:-1])


def escape_repl(m: Match[str], prefix: str) -> str:
    """Translate a string escape sequence, e.g. \t -> the tab character.

    Assume that the Match object is from escape_re.
    """

    seq = m.group(1)
    if len(seq) == 1 and seq in escape_map:
        # Single-character escape sequence, e.g. \n.
        return escape_map[seq]
    elif seq.startswith('x'):
        # Hexadecimal sequence \xNN.
        return chr(int(seq[1:], 16))
    elif seq.startswith('u'):
        # Unicode sequence \uNNNN.
        if 'b' not in prefix and 'B' not in prefix:
            return chr(int(seq[1:], 16))
        else:
            return '\\' + seq
    else:
        # Octal sequence.
        ord = int(seq, 8)
        if 'b' in prefix and 'B' in prefix:
            # Make sure code is no larger than 255 for bytes literals.
            ord = ord % 256
        return chr(ord)


class Lexer:
    """Lexical analyzer."""

    i = 0      # Current string index (into s)
    s = ''     # The string being analyzed
    line = 0   # Current line number
    pre_whitespace = ''     # Whitespace and comments before the next token
    enc = ''                # Encoding

    # Generated tokens
    tok = None  # type: List[Token]

    # Table from byte character value to lexer method. E.g. entry at ord('0')
    # contains the method lex_number().
    map = None  # type: Dict[str, Callable[[], None]]

    # Indent levels of currently open blocks, in spaces.
    indents = None  # type: List[int]

    # Open ('s, ['s and {'s without matching closing bracket; used for ignoring
    # newlines within parentheses/brackets.
    open_brackets = None  # type: List[str]

    pyversion = defaults.PYTHON3_VERSION

    # Ignore errors on these lines (defined using '# type: ignore').
    ignored_lines = None  # type: Set[int]

    def __init__(self, pyversion: Tuple[int, int] = defaults.PYTHON3_VERSION,
                 is_stub_file: bool = False) -> None:
        self.map = {}
        self.tok = []
        self.indents = [0]
        self.open_brackets = []
        self.pyversion = pyversion
        self.is_stub_file = is_stub_file
        self.ignored_lines = set()
        # Fill in the map from valid character codes to relevant lexer methods.
        extra_misc = '' if pyversion[0] >= 3 else '`'
        for seq, method in [('ABCDEFGHIJKLMNOPQRSTUVWXYZ', self.lex_name),
                            ('abcdefghijklmnopqrstuvwxyz_', self.lex_name),
                            ('0123456789', self.lex_number),
                            ('.', self.lex_number_or_dot),
                            (' ' + '\t' + '\x0c', self.lex_space),
                            ('"', self.lex_str_double),
                            ("'", self.lex_str_single),
                            ('\r' + '\n', self.lex_break),
                            (';', self.lex_semicolon),
                            (':', self.lex_colon),
                            ('#', self.lex_comment),
                            ('\\', self.lex_backslash),
                            ('([{', self.lex_open_bracket),
                            (')]}', self.lex_close_bracket),
                            ('-+*/<>%&|^~=!,@' + extra_misc, self.lex_misc)]:
            for c in seq:
                self.map[c] = method
        if pyversion[0] == 2:
            self.keywords = keywords_common | keywords2
            # Decimal/hex/octal/binary literal or integer complex literal
            self.number_exp1 = re.compile('(0[xXoObB][0-9a-fA-F]+|[0-9]+)[lL]?')

        if pyversion[0] == 3:
            self.keywords = keywords_common | keywords3
            self.number_exp1 = re.compile('0[xXoObB][0-9a-fA-F]+|[0-9]+')

    def lex(self, text: Union[str, bytes], first_line: int) -> None:
        """Lexically analyze a string, storing the tokens at the tok list."""
        self.i = 0
        self.line = first_line

        if isinstance(text, bytes):
            if text.startswith(b'\xef\xbb\xbf'):
                self.enc = 'utf8'
                bom = True
            else:
                self.enc, enc_line = find_python_encoding(text, self.pyversion)
                bom = False
            try:
                decoded_text = text.decode(self.enc)
            except UnicodeDecodeError as err:
                self.report_unicode_decode_error(err, text)
                return
            except LookupError:
                self.report_unknown_encoding(enc_line)
                return
            text = decoded_text
            if bom:
                self.add_token(Bom(text[0]))
        self.s = text

        # Parse initial indent; otherwise first-line indent would not generate
        # an error.
        self.lex_indent()

        # Use some local variables as a simple optimization.
        map = self.map
        default = self.unknown_character

        # Lex the file. Repeatedly call the lexer method for the current char.
        while self.i < len(text):
            # Get the character code of the next character to lex.
            c = text[self.i]
            # Dispatch to the relevant lexer method. This will consume some
            # characters in the text, add a token to self.tok and increment
            # self.i.
            map.get(c, default)()

        # Append a break if there is no statement/block terminator at the end
        # of input.
        if len(self.tok) > 0 and (not isinstance(self.tok[-1], Break) and
                                  not isinstance(self.tok[-1], Dedent)):
            self.add_token(Break(''))

        # Attach any dangling comments/whitespace to a final Break token.
        if self.tok and isinstance(self.tok[-1], Break):
            self.tok[-1].string += self.pre_whitespace
            self.pre_whitespace = ''

        # Close remaining open blocks with Dedent tokens.
        self.lex_indent()

        self.add_token(Eof(''))

    def report_unicode_decode_error(self, exc: UnicodeDecodeError, text: bytes) -> None:
        lines = text.splitlines()
        for line in lines:
            try:
                line.decode(self.enc)
            except UnicodeDecodeError as new_exc:
                exc = new_exc
                break
            self.line += 1
        else:
            self.line = 1
        self.add_token(
            LexError('', DECODE_ERROR,
                     "%r codec can't decode byte %d in column %d" % (
                         self.enc, line[exc.start], exc.start + 1)))
        self.add_token(Break(''))
        self.add_token(Eof(''))

    def report_unknown_encoding(self, encoding_line: int) -> None:
        self.line = encoding_line
        self.add_token(
            LexError('', DECODE_ERROR,
                     "Unknown encoding %r" % self.enc))
        self.add_token(Break(''))
        self.add_token(Eof(''))

    def lex_number_or_dot(self) -> None:
        """Analyse a token starting with a dot.

        It can be the member access operator, a float literal such as '.123',
        or an ellipsis (for Python 3 and for all stub files).
        """
        if self.is_at_number():
            self.lex_number()
        elif self.is_at_ellipsis():
            # '...' is valid in Python 2 as a token but it's use is limited to indexing.
            # Example: Tuple[int, ...] is valid in Python 2.
            self.lex_ellipsis()
        else:
            self.lex_misc()

    number_exp = re.compile(r'[0-9]|\.[0-9]')

    def is_at_number(self) -> bool:
        """Is the current location at a numeric literal?"""
        return self.match(self.number_exp) != ''

    ellipsis_exp = re.compile(r'\.\.\.')

    def is_at_ellipsis(self) -> bool:
        """Is the current location at a ellipsis '...'"""
        return self.match(self.ellipsis_exp) != ''

    # Regexps used by lex_number

    # NOTE: number_exp1 depends on Python version and is defined in __init__.

    # Float literal, e.g. '1.23' or '12e+34' or '1.2j'
    number_exp2 = re.compile(
        r'[0-9]*\.[0-9]*([eE][-+]?[0-9]+)?|[0-9]+[eE][-+]?[0-9]+')

    # Complex literal, e.g. '3j' or '1.5e2J'
    number_complex = re.compile(
        r'([0-9]*\.[0-9]*([eE][-+]?[0-9]+)?|[0-9]+([eE][-+]?[0-9]+)?)[jJ]')

    # These characters must not appear after a number literal.
    name_char_exp = re.compile('[a-zA-Z0-9_]')
    octal_int = re.compile('0+[1-9]')

    def lex_number(self) -> None:
        """Analyse an int or float literal.

        Assume that the current location points to one of them.
        """
        s1 = self.match(self.number_exp1)
        s2 = self.match(self.number_exp2)
        sc = self.match(self.number_complex)

        maxlen = max(len(s1), len(s2), len(sc))
        if self.name_char_exp.match(
                self.s[self.i + maxlen:self.i + maxlen + 1]) is not None:
            # Error: alphanumeric character after number literal.
            s3 = self.match(re.compile('[0-9][0-9a-zA-Z_]*'))
            maxlen = max(maxlen, len(s3))
            self.add_token(LexError(' ' * maxlen, NUMERIC_LITERAL_ERROR))
        elif len(s1) == maxlen:
            # Integer literal.
            if self.pyversion[0] >= 3 and self.octal_int.match(s1):
                # Python 2 style octal literal such as 0377 not supported in Python 3.
                self.add_token(LexError(s1, NUMERIC_LITERAL_ERROR))
            else:
                self.add_token(IntLit(s1))
        elif len(s2) == maxlen:
            # Float literal.
            self.add_token(FloatLit(s2))
        else:
            # Complex literal.
            self.add_token(ComplexLit(sc))

    def lex_ellipsis(self) -> None:
        self.add_token(EllipsisToken('...'))

    name_exp = re.compile('[a-zA-Z_][a-zA-Z0-9_]*')

    def lex_name(self) -> None:
        """Analyse a name.

        A name can be an identifier, a keyword or an alphabetical operator.
        Also deal with prefixed string literals such as r'...'.
        """
        s = self.match(self.name_exp)
        if s in self.keywords:
            self.add_token(Keyword(s))
        elif s in alpha_operators:
            self.add_token(Op(s))
        elif s in str_prefixes and self.match(re.compile('[a-zA-Z]+[\'"]')) != '':
            self.lex_prefixed_str(s)
        else:
            self.add_token(Name(s))

    # Regexps representing components of string literals

    # Initial part of a single-quoted literal, e.g. b'foo' or b'foo\\\n
    str_exp_single = re.compile(
        r"[a-zA-Z]*'([^'\\\r\n]|\\[^\r\n])*('|\\(\n|\r\n?))")
    # Non-initial part of a multiline single-quoted literal, e.g. foo'
    str_exp_single_multi = re.compile(
        r"([^'\\\r\n]|\\[^\r\n])*('|\\(\n|\r\n?))")
    # Initial part of a single-quoted raw literal, e.g. r'foo' or r'foo\\\n
    str_exp_raw_single = re.compile(
        r"[a-zA-Z]*'([^'\r\n\\]|\\'|\\[^\n\r])*('|\\(\n|\r\n?))")
    # Non-initial part of a raw multiline single-quoted literal, e.g. foo'
    str_exp_raw_single_multi = re.compile(
        r"([^'\r\n]|'')*('|\\(\n|\r\n?))")

    # Start of a ''' literal, e.g. b'''
    str_exp_single3 = re.compile("[a-z]*'''")
    # End of a ''' literal, e.g. foo'''
    str_exp_single3end = re.compile(r"([^\n\r\\]|\\[^\n\r])*?'''")

    # The following are similar to above (but use double quotes).

    str_exp_double = re.compile(
        r'[a-z]*"([^"\\\r\n]|\\[^\r\n])*("|\\(\n|\r\n?))')
    str_exp_double_multi = re.compile(
        r'([^"\\\r\n]|\\[^\r\n])*("|\\(\n|\r\n?))')
    str_exp_raw_double = re.compile(
        r'[a-z]*"([^"\r\n\\]|\\"|\\[^\n\r])*("|\\(\n|\r\n?))')
    str_exp_raw_double_multi = re.compile(
        r'([^"\r\n]|"")*("|\\(\n|\r\n?))')

    str_exp_double3 = re.compile('[a-z]*"""')
    str_exp_double3end = re.compile(r'([^\n\r\\]|\\[^\n\r])*?"""')

    def lex_str_single(self) -> None:
        """Analyse single-quoted string literal"""
        self.lex_str(self.str_exp_single, self.str_exp_single_multi,
                     self.str_exp_single3, self.str_exp_single3end)

    def lex_str_double(self) -> None:
        """Analyse double-quoted string literal"""
        self.lex_str(self.str_exp_double, self.str_exp_double_multi,
                     self.str_exp_double3, self.str_exp_double3end)

    def lex_prefixed_str(self, prefix: str) -> None:
        """Analyse a string literal with a prefix, such as r'...'."""
        s = self.match(re.compile('[a-zA-Z]+[\'"]'))
        if s.endswith("'"):
            re1 = self.str_exp_single
            re2 = self.str_exp_single_multi
            if 'r' in prefix or 'R' in prefix:
                re1 = self.str_exp_raw_single
                re2 = self.str_exp_raw_single_multi
            self.lex_str(re1, re2, self.str_exp_single3,
                         self.str_exp_single3end, prefix)
        else:
            re1 = self.str_exp_double
            re2 = self.str_exp_double_multi
            if 'r' in prefix or 'R' in prefix:
                re1 = self.str_exp_raw_double
                re2 = self.str_exp_raw_double_multi
            self.lex_str(re1, re2, self.str_exp_double3,
                         self.str_exp_double3end, prefix)

    def lex_str(self, regex: Pattern[str], re2: Pattern[str],
                re3: Pattern[str], re3end: Pattern[str],
                prefix: str = '') -> None:
        """Analyse a string literal described by regexps.

        Assume that the current location is at the beginning of the
        literal. The arguments re3 and re3end describe the
        corresponding triple-quoted literals.
        """
        s3 = self.match(re3)
        if s3 != '':
            # Triple-quoted string literal.
            self.lex_triple_quoted_str(re3end, prefix)
        else:
            # Single or double quoted string literal.
            s = self.match(regex)
            if s != '':
                if s.endswith('\n') or s.endswith('\r'):
                    self.lex_multiline_string_literal(re2, s)
                else:
                    if 'b' in prefix or 'B' in prefix:
                        self.add_token(BytesLit(s))
                    elif 'u' in prefix or 'U' in prefix:
                        self.add_token(UnicodeLit(s))
                    else:
                        self.add_token(StrLit(s))
            else:
                # Unterminated string literal.
                s = self.match(re.compile('[^\\n\\r]*'))
                self.add_token(LexError(s, UNTERMINATED_STRING_LITERAL))

    def lex_triple_quoted_str(self, re3end: Pattern[str], prefix: str) -> None:
        line = self.line
        ss = self.s[self.i:self.i + len(prefix) + 3]
        self.i += len(prefix) + 3
        while True:
            m = re3end.match(self.s, self.i)
            if m is not None:
                break
            m = re.match('[^\\n\\r]*(\\n|\\r\\n?)', self.s[self.i:])
            if m is None:
                self.add_special_token(
                    LexError(ss, UNTERMINATED_STRING_LITERAL), line, 0)
                return
            s = m.group(0)
            ss += s
            self.line += 1
            self.i += len(s)
        lit = None  # type: Token
        if 'b' in prefix or 'B' in prefix:
            lit = BytesLit(ss + m.group(0))
        elif 'u' in prefix or 'U' in prefix:
            lit = UnicodeLit(ss + m.group(0))
        else:
            lit = StrLit(ss + m.group(0))
        self.add_special_token(lit, line, len(m.group(0)))

    def lex_multiline_string_literal(self, re_end: Pattern[str],
                                     prefix: str) -> None:
        """Analyze multiline single/double-quoted string literal.

        Use explicit \ for line continuation.
        """
        line = self.line
        self.i += len(prefix)
        ss = prefix
        while True:
            m = self.match(re_end)
            if m == '':
                self.add_special_token(
                    LexError(ss, UNTERMINATED_STRING_LITERAL), line, 0)
                return
            ss += m
            self.line += 1
            self.i += len(m)
            if not m.endswith('\n') and not m.endswith('\r'): break
        self.add_special_token(StrLit(ss), line, 0)  # TODO bytes

    comment_exp = re.compile(r'#[^\n\r]*')

    def lex_comment(self) -> None:
        """Analyze a comment."""
        s = self.match(self.comment_exp)
        self.add_pre_whitespace(s)

    backslash_exp = re.compile(r'\\(\n|\r\n?)')

    def lex_backslash(self) -> None:
        s = self.match(self.backslash_exp)
        if s != '':
            self.add_pre_whitespace(s)
            self.line += 1
        else:
            self.add_token(LexError('\\', INVALID_BACKSLASH))

    space_exp = re.compile(r'[ \t\x0c]*')
    indent_exp = re.compile(r'[ \t]*[#\n\r]?')

    def lex_space(self) -> None:
        """Analyze a run of whitespace characters (within a line, not indents).

        Only store them in self.pre_whitespace.
        """
        s = self.match(self.space_exp)
        self.add_pre_whitespace(s)

    comment_or_newline = '#' + '\n' + '\r'  # type: str

    def lex_indent(self) -> None:
        """Analyze whitespace chars at the beginning of a line (indents)."""
        s = self.match(self.indent_exp)
        while True:
            s = self.match(self.indent_exp)
            if s == '' or s[-1] not in self.comment_or_newline:
                break
            # Empty line (whitespace only or comment only).
            self.add_pre_whitespace(s[:-1])
            if s[-1] == '#':
                self.lex_comment()
            else:
                self.lex_break()
        indent = self.calc_indent(s)
        if indent == self.indents[-1]:
            # No change in indent: just whitespace.
            self.add_pre_whitespace(s)
        elif indent > self.indents[-1]:
            # An increased indent (new block).
            self.indents.append(indent)
            self.add_token(Indent(s))
        else:
            # Decreased indent (end of one or more blocks).
            pre = self.pre_whitespace
            self.pre_whitespace = ''
            while indent < self.indents[-1]:
                self.add_token(Dedent(''))
                self.indents.pop()
            self.pre_whitespace = pre
            self.add_pre_whitespace(s)
            if indent != self.indents[-1]:
                # Error: indent level does not match a previous indent level.
                self.add_token(LexError('', INVALID_DEDENT))

    def calc_indent(self, s: str) -> int:
        indent = 0
        for ch in s:
            if ch == ' ':
                indent += 1
            else:
                # Tab: 8 spaces (rounded to a multiple of 8).
                indent += 8 - indent % 8
        return indent

    break_exp = re.compile(r'\r\n|\r|\n|;')

    def lex_break(self) -> None:
        """Analyse a line break."""
        s = self.match(self.break_exp)
        last_tok = self.tok[-1] if self.tok else None
        if isinstance(last_tok, Break):
            was_semicolon = last_tok.string == ';'
            last_tok.string += self.pre_whitespace + s
            self.i += len(s)
            self.line += 1
            self.pre_whitespace = ''
            if was_semicolon:
                self.lex_indent()
        elif self.ignore_break():
            self.add_pre_whitespace(s)
            self.line += 1
        else:
            self.add_token(Break(s))
            self.line += 1
            self.lex_indent()

    def lex_semicolon(self) -> None:
        self.add_token(Break(';'))

    def lex_colon(self) -> None:
        self.add_token(Colon(':'))

    open_bracket_exp = re.compile('[[({]')

    def lex_open_bracket(self) -> None:
        s = self.match(self.open_bracket_exp)
        self.open_brackets.append(s)
        self.add_token(Punct(s))

    close_bracket_exp = re.compile('[])}]')

    open_bracket = {')': '(', ']': '[', '}': '{'}

    def lex_close_bracket(self) -> None:
        s = self.match(self.close_bracket_exp)
        if (self.open_brackets != []
                and self.open_bracket[s] == self.open_brackets[-1]):
            self.open_brackets.pop()
        self.add_token(Punct(s))

    def lex_misc(self) -> None:
        """Analyze a non-alphabetical operator or a punctuator."""
        s = ''
        t = None  # type: Any
        for re_list, type in [(operators, Op), (punctuators, Punct)]:
            for regexp in re_list:
                s2 = self.match(regexp)
                if len(s2) > len(s):
                    t = type
                    s = s2
        if s == '':
            # Could not match any token; report an invalid character. This is
            # reached at least if the current character is '!' not followed by
            # '='.
            self.add_token(LexError(self.s[self.i], INVALID_CHARACTER))
        else:
            if s == '<>':
                if self.pyversion[0] == 2:
                    s = '!='
                else:
                    self.add_token(Op('<'))
                    s = '>'
            self.add_token(t(s))

    def unknown_character(self) -> None:
        """Report an unknown character as a lexical analysis error."""
        self.add_token(LexError(self.s[self.i], INVALID_CHARACTER))

    # Utility methods

    def match(self, pattern: Pattern[str]) -> str:
        """Try to match a regular expression at current location.

        If the argument regexp is matched at the current location,
        return the matched string; otherwise return the empty string.
        """
        m = pattern.match(self.s, self.i)
        if m is not None:
            return m.group(0)
        else:
            return ''

    def add_pre_whitespace(self, s: str) -> None:
        """Record whitespace and comments before the next token.

        The accumulated whitespace/comments will be stored in the next token
        and then it will be cleared.

        This is needed for pretty-printing the original source code while
        preserving comments, indentation, whitespace etc.
        """
        self.pre_whitespace += s
        self.i += len(s)

    type_ignore_exp = re.compile(r'[ \t]*#[ \t]*type:[ \t]*ignore\b')

    def add_token(self, tok: Token) -> None:
        """Store a token.

        Update its line number and record preceding whitespace
        characters and comments.
        """
        if (tok.string == '' and not isinstance(tok, Eof)
                and not isinstance(tok, Break)
                and not isinstance(tok, LexError)
                and not isinstance(tok, Dedent)):
            raise ValueError('Empty token')
        tok.pre = self.pre_whitespace
        if self.type_ignore_exp.match(tok.pre):
            delta = 0
            if '\n' in tok.pre or '\r' in tok.pre:
                delta += 1
            self.ignored_lines.add(self.line - delta)
        tok.line = self.line
        self.tok.append(tok)
        self.i += len(tok.string)
        self.pre_whitespace = ''

    def add_special_token(self, tok: Token, line: int, skip: int) -> None:
        """Like add_token, but caller sets the number of chars to skip."""
        if (tok.string == '' and not isinstance(tok, Eof)
                and not isinstance(tok, Break)
                and not isinstance(tok, LexError)
                and not isinstance(tok, Dedent)):
            raise ValueError('Empty token')
        tok.pre = self.pre_whitespace
        tok.line = line
        self.tok.append(tok)
        self.i += skip
        self.pre_whitespace = ''

    def ignore_break(self) -> bool:
        """If the next token is a break, can we ignore it?"""
        if len(self.open_brackets) > 0 or len(self.tok) == 0:
            # Ignore break after open ( [ or { or at the beginning of file.
            return True
        else:
            # Ignore break after another break or dedent.
            t = self.tok[-1]
            return isinstance(t, Break) or isinstance(t, Dedent)


if __name__ == '__main__':
    # Lexically analyze a file and dump the tokens to stdout.
    import sys
    if len(sys.argv) != 2:
        print('Usage: lex.py FILE', file=sys.stderr)
        sys.exit(2)
    fnam = sys.argv[1]
    s = open(fnam, 'rb').read()
    for t in lex(s):
        print(t)



# Stubs for tempfile
# Ron Murawski <ron@horizonchess.com>

# based on http://docs.python.org/3.3/library/tempfile.html

from typing import Tuple, BinaryIO

# global variables
tempdir = ...  # type: str
template = ...  # type: str

# TODO text files

# function stubs
def TemporaryFile(
            mode: str = ..., buffering: int = ..., encoding: str = ...,
            newline: str = ..., suffix: str = ..., prefix: str = ...,
            dir: str = ...) -> BinaryIO:
    ...
def NamedTemporaryFile(
            mode: str = ..., buffering: int = ..., encoding: str = ...,
            newline: str = ..., suffix: str = ..., prefix: str = ...,
            dir: str = ..., delete=...) -> BinaryIO:
    ...
def SpooledTemporaryFile(
            max_size: int = ..., mode: str = ..., buffering: int = ...,
            encoding: str = ..., newline: str = ..., suffix: str = ...,
            prefix: str = ..., dir: str = ...) -> BinaryIO:
    ...

class TemporaryDirectory:
    name = ...  # type: str
    def __init__(self, suffix: str = ..., prefix: str = ...,
                 dir: str = ...) -> None: ...
    def cleanup(self) -> None: ...
    def __enter__(self) -> str: ...
    def __exit__(self, type, value, traceback) -> bool: ...

def mkstemp(suffix: str = ..., prefix: str = ..., dir: str = ...,
            text: bool = ...) -> Tuple[int, str]: ...
def mkdtemp(suffix: str = ..., prefix: str = ...,
            dir: str = ...) -> str: ...
def mktemp(suffix: str = ..., prefix: str = ..., dir: str = ...) -> str: ...
def gettempdir() -> str: ...
def gettempprefix() -> str: ...



# Generated by stubtool 0.1, DO NOT EDIT
# See https://github.com/o11c/stubtool
#
# Stubs for six.moves.urllib_request (Python 2.7)

from six.moves.urllib.request import (
    urlopen as urlopen,
    install_opener as install_opener,
    build_opener as build_opener,
    pathname2url as pathname2url,
    url2pathname as url2pathname,
    getproxies as getproxies,
    Request as Request,
    OpenerDirector as OpenerDirector,
    HTTPDefaultErrorHandler as HTTPDefaultErrorHandler,
    HTTPRedirectHandler as HTTPRedirectHandler,
    HTTPCookieProcessor as HTTPCookieProcessor,
    ProxyHandler as ProxyHandler,
    BaseHandler as BaseHandler,
    HTTPPasswordMgr as HTTPPasswordMgr,
    HTTPPasswordMgrWithDefaultRealm as HTTPPasswordMgrWithDefaultRealm,
    AbstractBasicAuthHandler as AbstractBasicAuthHandler,
    HTTPBasicAuthHandler as HTTPBasicAuthHandler,
    ProxyBasicAuthHandler as ProxyBasicAuthHandler,
    AbstractDigestAuthHandler as AbstractDigestAuthHandler,
    HTTPDigestAuthHandler as HTTPDigestAuthHandler,
    ProxyDigestAuthHandler as ProxyDigestAuthHandler,
    HTTPHandler as HTTPHandler,
    HTTPSHandler as HTTPSHandler,
    FileHandler as FileHandler,
    FTPHandler as FTPHandler,
    CacheFTPHandler as CacheFTPHandler,
    UnknownHandler as UnknownHandler,
    HTTPErrorProcessor as HTTPErrorProcessor,
    urlretrieve as urlretrieve,
    urlcleanup as urlcleanup,
    URLopener as URLopener,
    FancyURLopener as FancyURLopener,
    proxy_bypass as proxy_bypass,
)



# Stubs for email.mime.image (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from email.mime.nonmultipart import MIMENonMultipart

class MIMEImage(MIMENonMultipart):
    def __init__(self, _imagedata, _subtype=..., _encoder=..., **_params) -> None: ...



# Stubs for pickle

from typing import Any, IO, Union, Tuple, Callable, Optional, Iterator
# Imports used in type comments only.
from typing import Mapping  # noqa

HIGHEST_PROTOCOL = ...  # type: int
DEFAULT_PROTOCOL = ...  # type: int


def dump(obj: Any, file: IO[bytes], protocol: int = None, *,
         fix_imports: bool = ...) -> None: ...


def dumps(obj: Any, protocol: int = ..., *,
          fix_imports: bool = ...) -> bytes: ...


def loads(bytes_object: bytes, *, fix_imports: bool = ...,
          encoding: str = ..., errors: str = ...) -> Any: ...


def load(file: IO[bytes], *, fix_imports: bool = ..., encoding: str = ...,
         errors: str = ...) -> Any: ...


class PickleError(Exception):
    pass


class PicklingError(PickleError):
    pass


class UnpicklingError(PickleError):
    pass


_reducedtype = Union[str,
                     Tuple[Callable[..., Any], Tuple],
                     Tuple[Callable[..., Any], Tuple, Any],
                     Tuple[Callable[..., Any], Tuple, Any,
                           Optional[Iterator]],
                     Tuple[Callable[..., Any], Tuple, Any,
                           Optional[Iterator], Optional[Iterator]]]


class Pickler:
    dispatch_table = ...  # type: Mapping[type, Callable[[Any], _reducedtype]]

    def __init__(self, file: IO[bytes], protocol: int = None, *,
                 fix_imports: bool = ...) -> None: ...

    def dump(self, obj: Any) -> None: ...

    def persistent_id(self, obj: Any) -> Any: ...


class Unpickler:
    def __init__(self, file: IO[bytes], *, fix_imports: bool = ...,
                 encoding: str = ..., errors: str = ...) -> None: ...

    def load(self) -> Any: ...

    def persistent_load(self, pid: Any) -> Any: ...

    def find_class(self, module: str, name: str) -> Any: ...



# Stubs for gzip (Python 3.5)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any
import _compression

def open(filename, mode='', compresslevel=9, encoding=None, errors=None, newline=None): ...

class _PaddedFile:
    file = ... # type: Any
    def __init__(self, f, prepend=b''): ...
    def read(self, size): ...
    def prepend(self, prepend=b''): ...
    def seek(self, off): ...
    def seekable(self): ...

class GzipFile(_compression.BaseStream):
    myfileobj = ... # type: Any
    mode = ... # type: Any
    name = ... # type: Any
    compress = ... # type: Any
    fileobj = ... # type: Any
    def __init__(self, filename=None, mode=None, compresslevel=9, fileobj=None, mtime=None): ...
    @property
    def filename(self): ...
    @property
    def mtime(self): ...
    crc = ... # type: Any
    def write(self, data): ...
    def read(self, size=-1): ...
    def read1(self, size=-1): ...
    def peek(self, n): ...
    @property
    def closed(self): ...
    def close(self): ...
    def flush(self, zlib_mode=...): ...
    def fileno(self): ...
    def rewind(self): ...
    def readable(self): ...
    def writable(self): ...
    def seekable(self): ...
    def seek(self, offset, whence=...): ...
    def readline(self, size=-1): ...

class _GzipReader(_compression.DecompressReader):
    def __init__(self, fp): ...
    def read(self, size=-1): ...

def compress(data, compresslevel=9): ...
def decompress(data): ...



# Stubs for tarfile (Python 2)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class TarError(Exception): ...
class ExtractError(TarError): ...
class ReadError(TarError): ...
class CompressionError(TarError): ...
class StreamError(TarError): ...
class HeaderError(TarError): ...
class EmptyHeaderError(HeaderError): ...
class TruncatedHeaderError(HeaderError): ...
class EOFHeaderError(HeaderError): ...
class InvalidHeaderError(HeaderError): ...
class SubsequentHeaderError(HeaderError): ...

class _LowLevelFile:
    fd = ... # type: Any
    def __init__(self, name, mode) -> None: ...
    def close(self): ...
    def read(self, size): ...
    def write(self, s): ...

class _Stream:
    name = ... # type: Any
    mode = ... # type: Any
    comptype = ... # type: Any
    fileobj = ... # type: Any
    bufsize = ... # type: Any
    buf = ... # type: Any
    pos = ... # type: Any
    closed = ... # type: Any
    zlib = ... # type: Any
    crc = ... # type: Any
    dbuf = ... # type: Any
    cmp = ... # type: Any
    def __init__(self, name, mode, comptype, fileobj, bufsize) -> None: ...
    def __del__(self): ...
    def write(self, s): ...
    def close(self): ...
    def tell(self): ...
    def seek(self, pos=...): ...
    def read(self, size=...): ...

class _StreamProxy:
    fileobj = ... # type: Any
    buf = ... # type: Any
    def __init__(self, fileobj) -> None: ...
    def read(self, size): ...
    def getcomptype(self): ...
    def close(self): ...

class _BZ2Proxy:
    blocksize = ... # type: Any
    fileobj = ... # type: Any
    mode = ... # type: Any
    name = ... # type: Any
    def __init__(self, fileobj, mode) -> None: ...
    pos = ... # type: Any
    bz2obj = ... # type: Any
    buf = ... # type: Any
    def init(self): ...
    def read(self, size): ...
    def seek(self, pos): ...
    def tell(self): ...
    def write(self, data): ...
    def close(self): ...

class _FileInFile:
    fileobj = ... # type: Any
    offset = ... # type: Any
    size = ... # type: Any
    sparse = ... # type: Any
    position = ... # type: Any
    def __init__(self, fileobj, offset, size, sparse=...) -> None: ...
    def tell(self): ...
    def seek(self, position): ...
    def read(self, size=...): ...
    def readnormal(self, size): ...
    def readsparse(self, size): ...
    def readsparsesection(self, size): ...

class ExFileObject:
    blocksize = ... # type: Any
    fileobj = ... # type: Any
    name = ... # type: Any
    mode = ... # type: Any
    closed = ... # type: Any
    size = ... # type: Any
    position = ... # type: Any
    buffer = ... # type: Any
    def __init__(self, tarfile, tarinfo) -> None: ...
    def read(self, size=...): ...
    def readline(self, size=...): ...
    def readlines(self): ...
    def tell(self): ...
    def seek(self, pos, whence=...): ...
    def close(self): ...
    def __iter__(self): ...

class TarInfo:
    name = ... # type: Any
    mode = ... # type: Any
    uid = ... # type: Any
    gid = ... # type: Any
    size = ... # type: Any
    mtime = ... # type: Any
    chksum = ... # type: Any
    type = ... # type: Any
    linkname = ... # type: Any
    uname = ... # type: Any
    gname = ... # type: Any
    devmajor = ... # type: Any
    devminor = ... # type: Any
    offset = ... # type: Any
    offset_data = ... # type: Any
    pax_headers = ... # type: Any
    def __init__(self, name=...) -> None: ...
    path = ... # type: Any
    linkpath = ... # type: Any
    def get_info(self, encoding, errors): ...
    def tobuf(self, format=..., encoding=..., errors=...): ...
    def create_ustar_header(self, info): ...
    def create_gnu_header(self, info): ...
    def create_pax_header(self, info, encoding, errors): ...
    @classmethod
    def create_pax_global_header(cls, pax_headers): ...
    @classmethod
    def frombuf(cls, buf): ...
    @classmethod
    def fromtarfile(cls, tarfile): ...
    def isreg(self): ...
    def isfile(self): ...
    def isdir(self): ...
    def issym(self): ...
    def islnk(self): ...
    def ischr(self): ...
    def isblk(self): ...
    def isfifo(self): ...
    def issparse(self): ...
    def isdev(self): ...

class TarFile:
    debug = ... # type: Any
    dereference = ... # type: Any
    ignore_zeros = ... # type: Any
    errorlevel = ... # type: Any
    format = ... # type: Any
    encoding = ... # type: Any
    errors = ... # type: Any
    tarinfo = ... # type: Any
    fileobject = ... # type: Any
    mode = ... # type: Any
    name = ... # type: Any
    fileobj = ... # type: Any
    pax_headers = ... # type: Any
    closed = ... # type: Any
    members = ... # type: Any
    offset = ... # type: Any
    inodes = ... # type: Any
    firstmember = ... # type: Any
    def __init__(self, name=..., mode=..., fileobj=..., format=..., tarinfo=..., dereference=..., ignore_zeros=..., encoding=..., errors=..., pax_headers=..., debug=..., errorlevel=...) -> None: ...
    posix = ... # type: Any
    @classmethod
    def open(cls, name=..., mode=..., fileobj=..., bufsize=..., **kwargs): ...
    @classmethod
    def taropen(cls, name, mode=..., fileobj=..., **kwargs): ...
    @classmethod
    def gzopen(cls, name, mode=..., fileobj=..., compresslevel=..., **kwargs): ...
    @classmethod
    def bz2open(cls, name, mode=..., fileobj=..., compresslevel=..., **kwargs): ...
    OPEN_METH = ... # type: Any
    def close(self): ...
    def getmember(self, name): ...
    def getmembers(self): ...
    def getnames(self): ...
    def gettarinfo(self, name=..., arcname=..., fileobj=...): ...
    def list(self, verbose=...): ...
    def add(self, name, arcname=..., recursive=..., exclude=..., filter=...): ...
    def addfile(self, tarinfo, fileobj=...): ...
    def extractall(self, path=..., members=...): ...
    def extract(self, member, path=...): ...
    def extractfile(self, member): ...
    def makedir(self, tarinfo, targetpath): ...
    def makefile(self, tarinfo, targetpath): ...
    def makeunknown(self, tarinfo, targetpath): ...
    def makefifo(self, tarinfo, targetpath): ...
    def makedev(self, tarinfo, targetpath): ...
    def makelink(self, tarinfo, targetpath): ...
    def chown(self, tarinfo, targetpath): ...
    def chmod(self, tarinfo, targetpath): ...
    def utime(self, tarinfo, targetpath): ...
    def next(self): ...
    def __iter__(self): ...
    def __enter__(self): ...
    def __exit__(self, type, value, traceback): ...

class TarIter:
    tarfile = ... # type: Any
    index = ... # type: Any
    def __init__(self, tarfile) -> None: ...
    def __iter__(self): ...
    def next(self): ...

class _section:
    offset = ... # type: Any
    size = ... # type: Any
    def __init__(self, offset, size) -> None: ...
    def __contains__(self, offset): ...

class _data(_section):
    realpos = ... # type: Any
    def __init__(self, offset, size, realpos) -> None: ...

class _hole(_section): ...

class _ringbuffer(list):
    idx = ... # type: Any
    def __init__(self) -> None: ...
    def find(self, offset): ...

class TarFileCompat:
    tarfile = ... # type: Any
    def __init__(self, file, mode=..., compression=...) -> None: ...
    def namelist(self): ...
    def infolist(self): ...
    def printdir(self): ...
    def testzip(self): ...
    def getinfo(self, name): ...
    def read(self, name): ...
    def write(self, filename, arcname=..., compress_type=...): ...
    def writestr(self, zinfo, bytes): ...
    def close(self): ...

def is_tarfile(name): ...



from typing import Callable, Any, TypeVar

__all__ = ... # type: str

_T = TypeVar('_T')

def coroutine(func: _T) -> _T: ...
def iscoroutinefunction(func: Callable[..., Any]) -> bool: ...
def iscoroutine(obj: Any) -> bool: ...



from typing import Any, IO

def dumps(obj: Any) -> str: ...
def dump(obj: Any, fp: IO[str], *args: Any, **kwds: Any) -> None: ...
def loads(s: str, **kwds: Any) -> Any: ...
def load(fp: IO[str]) -> Any: ...

from .scanner import JSONDecodeError
from .decoder import JSONDecoder
from .encoder import JSONEncoder, JSONEncoderForHTML



# Stubs for gettext (Python 3.4)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any

class NullTranslations:
    def __init__(self, fp=...) -> None: ...
    def add_fallback(self, fallback): ...
    def gettext(self, message): ...
    def lgettext(self, message): ...
    def ngettext(self, msgid1, msgid2, n): ...
    def lngettext(self, msgid1, msgid2, n): ...
    def info(self): ...
    def charset(self): ...
    def output_charset(self): ...
    def set_output_charset(self, charset): ...
    def install(self, names=...): ...

class GNUTranslations(NullTranslations):
    LE_MAGIC = ...  # type: Any
    BE_MAGIC = ...  # type: Any
    def lgettext(self, message): ...
    def lngettext(self, msgid1, msgid2, n): ...
    def gettext(self, message): ...
    def ngettext(self, msgid1, msgid2, n): ...

def find(domain, localedir=..., languages=..., all=...): ...
def translation(domain, localedir=..., languages=..., class_=..., fallback=...,
                codeset=...): ...
def install(domain, localedir=..., codeset=..., names=...): ...
def textdomain(domain=...): ...
def bindtextdomain(domain, localedir=...): ...
def dgettext(domain, message): ...
def dngettext(domain, msgid1, msgid2, n): ...
def gettext(message): ...
def ngettext(msgid1, msgid2, n): ...

Catalog = ...  # type: Any
